/**
 * Handles all the remote requests. Provides a jQuery.ajax-like interface for requests. See http://docs.jquery.com.
 *
 * When making ajax requests an 'authorisation' process will be used to determine
 * the method and settings required to make and authorised remote requests. The process is:
 *
 * jQuery - If the request url is part of the same web-app as the gadget
 * trustedApps - If the request url is not part of the same web-app
 * OAuth - If the request url is not part of the same web-app and anonymous access is not allowed.
 * anonymous - If the request url is not part of the same web-app and anonymous access is allowed
 *
 * @param {function(Object)} ajaxOptions - a restricted set of jQuery's ajaxOptions
 */
jQuery.ajax = function (options, queue) {

    var paramWhiteList = [
            /* We do not support all the jQuery ajax options as they need to be available in both jQuery and makeRequest */
            "success", "error", "complete", "send", "url", "data", "dataType", "headers", "type", "global"
        ],

        requestParams = AG.copyObject(jQuery.ajaxSettings),
        userParams = AG.copyObject(options, true),
        authQueue = queue || [AG.AJAX.jQuery, AG.AJAX.trustedApps, AG.AJAX.OAuth, AG.AJAX.anonymous],

        /* Make a request using first request mechanism from authQueue */
        request = authQueue.splice(0,1)[0];

    jQuery.extend(requestParams, userParams, {

        send: function () {

            if (options.send) {
                options.send.apply(this, arguments);
            }

            if (options.global) {
                jQuery.event.trigger("ajaxStart", arguments);
            }
        },

        complete: function () {

            if (options.complete) {
                options.complete.apply(this, arguments);
            }

            if (options.global) {
                jQuery.event.trigger("ajaxStop", arguments);
            }
        },

        error: function () {

            if (options.error) {
                options.error.apply(this, arguments);
            }

            if (options.global) {
                jQuery.event.trigger("ajaxError", arguments);
            }
        },

        success: function () {

            if (options.success) {
                options.success.apply(this, arguments);
            }

            if (options.global) {
                jQuery.event.trigger("ajaxSuccess", arguments);
            }
        },

        authError: function () {
            if (authQueue.length > 0) {
                jQuery.ajax(options, authQueue);
            }
        }
    });

    jQuery.each(paramWhiteList, function (i, name) {
        if (jQuery.inArray(name, paramWhiteList) === -1) {
            console.warn("jQuery.ajax: param [" + name + "] is invalid. Ignoring...");
        }
    });

    request(requestParams);
};
    