jQuery.namespace("AG.AJAX.trustedApps");

/**
 * Makes a request using gadgets.io.makeRequest but with jQuery.ajax formatted paramaters
 *
 * @param {object} options - jQuery ajax options 
 */
AG.AJAX.trustedApps = function (options){

    var params;

    // Trusted apps is Atlassian ONLY

    if (!AG.isAtlassianContainer()) {

        if (options.authError) {
            options.authError();
        }

    } else {

        params = AG.mapToMakeRequestOptions(options);

        options.send();

        gadgets.io.makeRequest(params.url, function (response) {
            AG.AJAX.trustedApps.handleResponse(options, response);
        }, params);

    }
};

/**
 * Handles makeRequest response
 *
 * @param options - ajax settings
 * @param {object} response - makeRequest response  object
 */
AG.AJAX.trustedApps.handleResponse = function (options, response) {

    /* This header was only passed through to the client in AG.3.0. We use this header to determine if the response
        has accessed authenticated data using Trusted Apps or it has just returned anonymous data. If the header does not
        exsist we cannot tell if trusted apps exist so throw an authError anyway. */
    var trustedAppsHeader = response.headers["x-seraph-trusted-app-status"];

    if (trustedAppsHeader && trustedAppsHeader[0] === "OK") {
        AG.AJAX.makeRequestToJQueryAjaxCallbackTransformer(options, response);
    } else if (options.authError) {
        options.authError();
    }
};