/*
 * This file drives the install page
 */
(function($) {

    // Wait for UPM to load
    var upmReady = function() {

        // Stop waiting for UPM to load
        $(window).unbind('upmready', upmReady);

        var $plugins,
            isIE = upm.browser.isIE,
            isIE7 = upm.browser.isIE7,
            isIE8AndLower = upm.browser.isIE8AndLower,
            hash = upm.getLocationHash('install'),
            mostRequestedLoaded,
            listOptions = {isExpandable: true, isInstalledList: false, className: 'install'};

        /**
         * Removes the "Search" option from the Install tab dropdown and resets the search box
         * @method removeSearchOption
         */
        function removeSearchOption() {
            // remove search option from dropdown
            AJS.$('#upm-search-option').remove();
            // clear search term, if there is one
            AJS.$('#upm-install-search-box').val('').blur();
        }

        /**
         * Appends a group of plugin elements to a specified container
         * @method buildPluginElements
         * @param {Array} plugins An array of plugin objects
         * @param {HTMLElement} container Dom element to append to
         * @param {Object} options Plugin options
         */
        function buildPluginElements(plugins, container, options) {
            for (var i = 0, len = plugins.length; i < len; i++) {
                container.append(buildPluginElement(plugins[i], options));
            }
        }

        function buildPluginActionButtons(pluginInfo) {
            buildPluginActionButtons.template = buildPluginActionButtons.template || AJS.$(AJS.$('#upm-plugin-actions-template').html());
            var buttons = buildPluginActionButtons.template.clone();

            if (pluginInfo.installed || pluginInfo.userInstalled) {
                if (pluginInfo.links['manage']) {
                    AJS.$('a.upm-manage-plugin', buttons).attr('href', pluginInfo.links.manage);
                    AJS.$('a.upm-manage-plugin', buttons).closest('ul.toolbar-group').removeClass('hidden');
                } else {
                    AJS.$('span.already-installed-message', buttons).removeClass('hidden');
                }
                AJS.$('a.upm-install', buttons).closest('ul.toolbar-group').remove();
                AJS.$('a.upm-buy', buttons).closest('ul.toolbar-group').remove();
                AJS.$('a.upm-try', buttons).closest('ul.toolbar-group').remove();
            } else {
                if (pluginInfo.links['manage']) {
                    AJS.$('a.upm-manage-plugin', buttons).closest('ul.toolbar-group').remove();
                }
                AJS.$('span.already-installed-message', buttons).remove();
                if (!pluginInfo.links['binary']) {
                    AJS.$('a.upm-install', buttons).closest('ul.toolbar-group').addClass('hidden');
                }

                if (pluginInfo.links['request']) {
                    AJS.$('a.upm-request', buttons).closest('ul.toolbar-group').removeClass('hidden').click(function(e) {
                        e.preventDefault();
                        showRequestSubmitDialog(e);
                    });

                    if (pluginInfo.requests && pluginInfo.requests.length) {
                        AJS.$(pluginInfo.requests).each(function(i) {
                            if (pluginInfo.requests[i].user && pluginInfo.requests[i].user.username === AJS.params.upmCurrentUsername) {
                                AJS.$('a.upm-request', buttons).text(AJS.params['upm.plugin.action.request.update']);
                                AJS.$('<input>')
                                    .attr('type', 'hidden')
                                    .attr('value', pluginInfo.requests[i].message || '')
                                    .attr('class', 'upm-request-existing-message')
                                    .appendTo(buttons);
                            }
                        });
                    }
                }

                if (!pluginInfo.links['dismiss-request']) {
                    AJS.$('a.upm-dismiss-request', buttons).closest('ul.toolbar-group').remove();
                }

                // Change the install button to a download button if the plugin isn't deployable. This includes if there is no binary url,
                // in which case clicking download will show a modal explaining what to do instead
                if (!upm.isPluginInstallable(pluginInfo)) {
                    if (pluginInfo.links['binary']) {
                        AJS.$('a.upm-install', buttons)
                            .removeClass('upm-install')
                            .addClass('upm-download')
                            .text(AJS.params['upmTextDownload'])
                            .closest('li.toolbar-item').removeClass('upm-install-item').addClass('upm-download-item');
                    }
                    AJS.$('a.upm-buy', buttons).closest('ul.toolbar-group').remove();
                    AJS.$('a.upm-try', buttons).closest('ul.toolbar-group').remove();
                    // non-deployable plugins cannot be sold on the marketplace
                } else if ((pluginInfo.marketplaceType === "MARKETPLACE" || pluginInfo.marketplaceType === "PAID_VIA_ATLASSIAN")
                		&& !isPurchasedAddonsPage()){
                    AJS.$('a.upm-install', buttons).closest('ul.toolbar-group').remove();
                    if (pluginInfo.links['binary']) {
                        AJS.$('a.upm-buy', buttons).closest('ul.toolbar-group').removeClass('hidden');
                        AJS.$('a.upm-try', buttons).closest('ul.toolbar-group').removeClass('hidden');
                    }
                }
            }
            AJS.$('a.upm-install', buttons).attr('id', 'upm-install-' + pluginInfo.key);
            AJS.$('a.upm-buy', buttons).attr('id', 'upm-install-' + pluginInfo.key);
            AJS.$('a.upm-try', buttons).attr('id', 'upm-install-' + pluginInfo.key);
            return buttons;
        }

        function getDisplayPricing(pricing) {
            return '$' + Math.floor(pricing.usdAmount);
        }

        function formatNumberWithCommas(number) {
            if (number) {
                return (number + '').replace(/(\d)(?=(\d\d\d)+(?!\d))/g, "$1,");
            } else {
                return 0;
            }
        }

        /**
         * Handle category link click
         * @param event
         */
        function onCategoryLinkClick(event) {
            var link = $(event.target),
                categoryName = link.data('categoryName');

            if ($('#upm-install-single').is(':visible')) {
                reloadFilters({type : 'featured', category : categoryName});
                $('#carousel, #upm-install-search-form-container, .fancy-select').show();
            } else if (getFilterOptions().type === 'search' || getFilterOptions().type === 'most-requested') {
                //must set the category this way because the filters take the previous value when handling a filter change
                AJS.$('#upm-install-category').attr('data-prev-value', categoryName);
                reloadFilters({type : 'featured'});
            } else {
                $('#upm-install-category .options li[data-value="' + categoryName + '"]').trigger('click');
            }

            event.preventDefault();
            event.stopPropagation();
        }

        /**
         * Build a single plugin element
         * @method buildPluginElement
         * @param {Object} plugin The plugin data representation
         * @param {Object} options Plugin options
         */
        function buildPluginElement(plugin, options) {
            buildPluginElement.template = buildPluginElement.template ||
                AJS.$(AJS.$('#upm-plugin-install-template').html());

            var pluginElement = buildPluginElement.template.clone();
            return populateRowTemplate(plugin, pluginElement, options);

        }

        /**
         * Take a template and put the data into it
         * @method populateRowTemplate
         * @param {Object} plugin The plugin data representation
         * @param {Object} plugin The pluginElement which is the row template itself (this is used for two row templates
         * @param {Object} options Plugin options
         */

        function populateRowTemplate(plugin, pluginElement, options) {

            var hash = upm.createHash(plugin.key, 'install'),
                updateLink = plugin.links['update-details'],
                categories,
                categoriesElement = pluginElement.find('.upm-plugin-categories'),
                nameLink = pluginElement.find('.upm-plugin-name-link'),
                pluginNameHtml = upm.html_sanitize(plugin.name, upm.htmlSanitizerUrlPolicy);

            pluginElement.attr('id', 'upm-plugin-' + hash);
            if (nameLink.length) {
                nameLink.html(pluginNameHtml);
            } else {
                pluginElement.find('.upm-plugin-name').html(pluginNameHtml);
            }

            pluginElement.find('input.upm-plugin-key').val(plugin.key);

            if (plugin.summary || plugin.description) {
                pluginElement.find('p.upm-plugin-summary')
                    .html(upm.html_sanitize(plugin.summary || plugin.description, upm.htmlSanitizerUrlPolicy))
                    .find('a').each(function() {
                        var el = AJS.$(this);
                        el.replaceWith(el.html());
                    });
            }

            if (plugin.vendor) {
                pluginElement.find('.upm-plugin-vendor').text(plugin.vendor.name);
            }
            pluginElement.find('.upm-plugin-download-count')
                .text(AJS.format(
                plugin.downloadCount == 1 ? AJS.params['upm.plugin.downloads.singular'] : AJS.params['upm.plugin.downloads.plural'],
                plugin.downloadCount ? formatNumberWithCommas(plugin.downloadCount) : 0));

            setStars(pluginElement, plugin);

            if (plugin.categories && plugin.categories.length) {
                if (plugin.categories.length > 3) {
                    categories = plugin.categories.slice(0, 3);
                } else {
                    categories = plugin.categories;
                }

                //hack to split up the categories and apply styling
                for (var i = 0; i < categories.length; i++) {
                    var categoryLink = AJS.$('<a class="category-link subtle-gray"></a>')
                        .data('categoryName', categories[i].name)
                        .attr('href', categories[i].links.self)
                        .text(categories[i].name);
                    categoriesElement.append(categoryLink);
                }

                categoriesElement.delegate('.category-link', 'click', onCategoryLinkClick);

            } else {
                pluginElement.find('.upm-plugin-categories').append("No Categories");
            }
            if (plugin.pricing) {
                pluginElement.find('.upm-plugin-price').append('<span class="subtle-green">' + getDisplayPricing(plugin.pricing) + '</span>');
            } else if (plugin.marketplaceType === "MARKETPLACE" || plugin.marketplaceType === "PAID_VIA_ATLASSIAN") {
                pluginElement.find('.upm-plugin-price').append(AJS.params['upm.plugin.pricing.marketplacePaid']);
            } else if (plugin.marketplaceType === "OFFMARKET" || plugin.marketplaceType === "PAID_VIA_VENDOR") {
                pluginElement.find('.upm-plugin-price').append(AJS.params['upm.plugin.pricing.paid']);
            } else {
                pluginElement.find('.upm-plugin-price').append(AJS.params['upm.plugin.pricing.free']);
            }

            if (plugin.links.binary) {
                AJS.$('input.upm-plugin-binary', pluginElement).val(plugin.links.binary);
            }
            if (plugin.links.details) {
                AJS.$('input.upm-plugin-homepage', pluginElement).val(plugin.links.details);
            }

            if (plugin.purchased && plugin.purchased.license) {
                pluginElement.find('.upm-plugin-purchased-license').text(upm.getLicenseDescription(plugin.purchased.license));
                if (plugin.purchased.license.error) {
                    pluginElement.find('.upm-plugin-purchased-license-error').text('(' +
                        upm.getLicenseStatusDescription(plugin.purchased.license) + ')');
                }
                pluginElement.find('.upm-plugin-purchased-date').text(plugin.purchased.license.creationDateString);
            }

            $('.upm-plugin-actions', pluginElement).replaceWith(buildPluginActionButtons(plugin));

            pluginElement.attr('data-link-self', upm.getPluginLink(plugin, options.isUpdatable));
            pluginElement.attr('data-link-modify', plugin.links['modify']);
            pluginElement.attr('data-link-delete', plugin.links['delete']);

            if (updateLink) {
                pluginElement.attr('data-link-update-details', updateLink);
            }

            if (plugin.links['dismiss-request']) {
                pluginElement.attr('dismiss-request-link', plugin.links['dismiss-request']);
            }

            upm.addPluginDataAttributes(pluginElement, plugin);

            var logoElement = AJS.$('<img src="" alt="" class="upm-plugin-logo" width="72">');
            if (plugin.logo) {
                logoElement.attr('src', plugin.logo.link);
                if (plugin.logo.width < plugin.logo.height) {
                    logoElement
                        .removeAttr('width')
                        .attr('height', 72);
                }
            } else {
                logoElement.attr('src', AJS.params['defaultPluginLogo'])
            }
            AJS.$('.upm-plugin-logo-container', pluginElement).append(logoElement);

            if (plugin.restartState) {
                pluginElement.addClass('installed-but-requires-restart');
            }


            if (plugin.requests && plugin.requests.length) {
                var requestCount = pluginElement.find('.request-count-message');
                requestCount.text(AJS.format(AJS.params['upm.requests.count.' + (plugin.requests.length > 1 ? 'plural' : 'singular')], plugin.requests.length));

                if((getFilterOptions().type === 'most-requested') || options.showRequestCount) {
                    requestCount.removeClass('hidden');
                }
            }

            return pluginElement;
        }

        /**
         * Creates and returns a plugin list element with text stating no installable plugins were found
         * that the user can install
         * @method buildEmptyPluginListForInstalling
         * @return {HTMLElement} Element representing empty plugin list
         */
        function buildEmptyPluginListForInstalling() {
            return isPurchasedAddonsPage() ? null :
                AJS.$('<p class="upm-no-matching-plugins"></p>').text(AJS.params.upmTextNotFoundInstall);
        }

        /**
         * Creates and returns a list of plugins for insertion into the dom
         * @method buildPluginList
         * @param {Array} plugins An array of plugin objects
         * @param {Object} options Whether individual plugins should be expandable to reveal plugin details
         * @return {HTMLElement} Element representing plugin list
         */
        function buildPluginList(plugins, options) {
            buildPluginList.template = buildPluginList.template || AJS.$(AJS.$('#upm-plugin-list-template').html());
            var listContainer = buildPluginList.template.clone(),
                list = AJS.$('.upm-plugin-list', listContainer);
            if ((plugins && plugins.length) || options.forceCreation) {
                if (options.isExpandable) {
                    list.addClass('expandable');
                }
                if (options.isInstalledList) {
                    list.addClass('upm-installed-list');
                }
                if (options.className) {
                    list.addClass(options.className);
                }
                buildPluginElements(plugins, list, options);
            } else {
                listContainer = buildEmptyPluginListForInstalling();
            }
            return listContainer;
        }

        /**
         * Gets a list of plugins from a given location, builds DOM elements and inserts them into a specified container.
         * Fires the JQuery event "pluginsLoaded.install" (with the Ajax response as a parameter) when the list has been
         * loaded.
         * @method loadPlugins
         * @param {String} url Location of the resource to hit to retrieve the list of plugins
         * @param {Object} listOptions Object defining list characteristics
         * @param {HTMLElement} listContainer Container to load plugin list into
         * @param {HTMLElement} parentContainer (optional) Container to mark as "loaded" when finished
         * @param {Function} callback (optional) Function call after finished loading
         */
        function loadPlugins(url, category, cost, listOptions, listContainer, parentContainer, callback) {
            var
            // check to see if there's existing pagination data, using the default value of 10 for local lists (in the resources)
                pagination = listContainer.data('pagination') || {
                    'start-index': 0,
                    'max-results': 10,
                    'category': category,
                    'cost': cost
                },
                // so that we can tell if we're starting from scratch or appending to an existing list
                isFreshLoad = (pagination['start-index'] === 0);

            parentContainer = parentContainer || listContainer;
            if (listContainer.hasClass('loading')) {
                // if we're already loading these plugins, don't do anything
                return;
            }
            if (isFreshLoad) {
                listContainer.addClass('loading');
            }

            AJS.$.ajax({
               url: url,
               type: 'get',
               cache: false,
               dataType: 'json',
               data: pagination,
               success: function(response) {
                   var plugins = response.plugins,
                       list,
                       seeMoreElement;

                   if (isFreshLoad) {
                       // create a whole new plugin list
                       listContainer.removeClass('loading').append(buildPluginList(plugins, listOptions));
                       list = AJS.$('div.upm-plugin-list', listContainer);
                   } else {
                       // append to an existing plugin list
                       list = AJS.$('div.upm-plugin-list', listContainer);
                       buildPluginElements(plugins, list, listOptions);
                   }
                   // set the start index to the number of existing plugin elements
                   pagination['start-index'] = AJS.$('div.upm-plugin', list).length;
                   parentContainer.addClass('loaded');

                   upm.trigger('pluginsLoaded.install', [response]);

                   if (upm.isUnknownProductVersion && !upm.isDevelopmentProductVersion) {
                       AJS.$('#upm-install-search-form-container').addClass('hidden');
                   }

                   seeMoreElement = AJS.$('a.upm-plugins-see-more', listContainer).removeClass('loading');
                   if (response.links.next) {
                       listContainer.addClass('has-more');
                       seeMoreElement.bind('click', function(e) {
                           var seeMoreElement = AJS.$('a.upm-plugins-see-more', listContainer);
                           e.preventDefault();
                           if (!seeMoreElement.hasClass('loading')) {
                               seeMoreElement
                                   .addClass('loading')
                                   .unbind('click');
                               loadPlugins(response.links.next, category, cost, listOptions, listContainer, parentContainer);
                           }
                       });
                   } else {
                       listContainer.removeClass('has-more');
                   }
                   // save the pagination data back to the container element
                   listContainer.data('pagination', pagination);

                   // will only be true if this is the "view requests" page
                   if (response.hasRequests) {
                       //ideally the existence of this button would be controlled by what exists in
                       //the plugin representation. however, due to some circular injection dependencies,
                       //this isn't very easy nor efficient. as a result, we'll unhide these buttons
                       //as such because we have requests to display.
                       if(!isUserFacing()) {
                           AJS.$('a.upm-dismiss-request').closest('ul.toolbar-group').removeClass('hidden');
                       }
                   }
                   
                   callback && callback();
               },
               error: function(request) {
                   listContainer.removeClass('loading');
                   AJS.$('a.upm-plugins-see-more', listContainer).removeClass('loading');
                   upm.handleAjaxError(upm.$messageContainer, request,'');
                   
                   callback && callback();
               }
           });
        }

        upm.reloadPlugins = function(callback) {
            changeDisplayedPlugins(true, callback);
        };
        
        function selectPlugin(url, callback) {
            var container = AJS.$('#upm-install-single'),
                parentContainer = AJS.$('#upm-install-container-single');

            if (url) {
                $('#upm-panel-install .upm-install-type, #carousel, #upm-install-search-form-container, .fancy-select, ' +
                    '#upm-install-container-most-requested, #upm-most-requested, #upm-find-more').fadeOut(300, function() {
                    container.empty();
                    loadSinglePlugin(url, container, parentContainer, callback);
                });
            }
        }

        function loadSinglePlugin(url, listContainer, parentContainer, callback) {
            parentContainer = parentContainer || listContainer;
            if (listContainer.hasClass('loading')) {
                // if we're already loading these plugins, don't do anything
                return;
            }

            listContainer.addClass('loading');

            AJS.$.ajax({
                url: url,
                type: 'get',
                dataType: 'json',
                success: function(plugin) {
                    listContainer.append(buildPluginList([plugin], {isExpandable: false, isInstalledList: false, className: 'install'}));

                    // expand plugin details
                    upm.buildPluginMarkup(listContainer.find('.upm-details'), plugin, 'install');
                    listContainer.find('.upm-plugin').addClass('expanded');
                    listContainer.removeClass('loading').fadeIn(300, function() {
                        $('html, body').animate({ scrollTop: 0 }, 200);
                        parentContainer.show();
                    });
                },
                error: function(request) {
                    reloadFilters({type : 'featured'});
                    doPushState();
                }
            });
        }

        function buildPluginRecommendations(link, detailsTemplate) {
            var recContainer = AJS.$('.upm-plugin-recommendations-container', detailsTemplate);
            if (isPurchasedAddonsPage()) {
                recContainer.addClass('loaded');
                return;
            }
            AJS.$.ajax({
                type: 'get',
                cache: false,
                url: link,
                dataType: 'json',
                success: function (recommendationsResponse) {
                    buildPluginRecommendations.template = buildPluginRecommendations.template || AJS.$(AJS.$('#upm-plugin-recommendation-item').html());
                    var recDiv = AJS.$('.upm-plugin-recommendations', detailsTemplate);
                    var recommendations =  recommendationsResponse.recommendations;

                    if(recommendations.length) {
                        AJS.$('.upm-plugin-recommendations-container', detailsTemplate).removeClass('hidden');

                        AJS.$.each(recommendations, function() {
                            var recItem = buildPluginRecommendations.template.clone(),
                                plugin = this;

                            AJS.$('.recommendation-name', recItem).text(plugin.name);

                            setStars(recItem, plugin);

                            var logoElement = AJS.$('.recommendation-logo-img', recItem);

                            if (plugin.logo) {
                                logoElement.attr('src', plugin.logo.link);
                                if (plugin.logo.width < plugin.logo.height) {
                                    logoElement
                                        .removeAttr('width')
                                        .attr('height', 56);
                                }
                            } else {
                                logoElement.attr('src', AJS.params['defaultPluginLogo'])
                            }

                            recItem.click(function() {
                                var webUrl = constructMarketplaceUri(upm.resources['available'], {restPluginKey : plugin.key}),
                                    newUrl = constructMarketplaceUri(isUserFacing() ? upm.resources['plugin-requests'] : upm.resources['marketplace'], {pluginKey : plugin.key}),
                                    singleContainer = AJS.$('#upm-install-single');

                                if (singleContainer.is(':visible')) {
                                    singleContainer.fadeOut(300, function() {
                                        singleContainer.empty();
                                        loadSinglePlugin(webUrl, singleContainer, AJS.$('#upm-install-container-single'));
                                    });
                                    upm.pushState({type: 'single', pluginKey: plugin.key}, newUrl, newUrl, true);
                                } else {
                                    selectPlugin(webUrl);
                                    upm.pushState({type: 'single', pluginKey: plugin.key}, newUrl, newUrl);
                                }
                            });

                            recDiv.append(recItem);
                        });
                    }

                    recContainer.addClass('loaded');
                },
                error: function() {
                    //don't display any error to the user...simply don't display any recommendations.
                    recContainer.addClass('loaded');
                }
            });
        }

        function setStars(container, plugin) {
            if (plugin.rating > 0) {
                container.find('.upm-plugin-rating')
                    .css('width', (plugin.rating * 15) + 'px')
                    .attr('title', AJS.format(AJS.params['upm.plugin.rating'], (plugin.rating + '').substr(0, 3), 4));
            }
            if (plugin.ratingCount > 0) {
                container.find('.upm-plugin-rating-count').text('(' + plugin.ratingCount + ')');
            }
        }

        function buildPluginRequests(plugin, pluginElement) {
            buildPluginRequests.template = buildPluginRequests.template || AJS.$(AJS.$('#upm-request-plugin-message').html());

            var  i,
                request,
                messages = [],
                message,
                html;

            for (i=0; i < plugin.requests.length; i++) {
                message = buildPluginRequests.template.clone();
                request = plugin.requests[i];

                //set the request author
                if (request.user.userProfileUri) {
                    message.find('.request-user .request-value')
                        .append($("<a></a>")
                            .attr('href', request.user.userProfileUri)
                            .text(request.user.displayName || request.user.username));
                } else {
                    message.find('.request-user .request-value').text(request.user.displayName || request.user.username);
                }

                //set the request timestamp
                message.find('.request-timestamp .request-value').text(upm.prettyDate(request.timestamp));

                //set the request message
                if (request.message) {
                    message.find('.request-message .request-value').text(request.message);
                } else {
                    message.find('.request-message').remove();
                }

                messages.push(message);
            }

            if (messages.length) {
                html = '';
                for (i = 0; i < messages.length; i++) {
                    html += messages[i].html();
                }

                upm.displayMessage(pluginElement, {
                    body: html,
                    type: 'info',
                    closeable: false
                });
            }
        }

        /**
         * Handle filter change
         * @param event
         */
        function onFilterChange(event) {
            event.preventDefault();
            changeDisplayedPlugins(false);
        }

        /**
         * Changes which plugins are displayed on the "Install" tab.
         * @method changeDisplayedPlugins
         */
        function changeDisplayedPlugins(skipPushState, callback) {
            var options = getFilterOptions(),
                container = AJS.$('#upm-install-' + options.type),
                parentContainer = AJS.$('#upm-install-container-' + options.type);

            AJS.$('#single-container-back-link').removeData('fromPluginSelection');

            changeFilterState(options);
            options = getFilterOptions();

            AJS.$('#upm-panel-install .upm-install-type').hide().removeClass('loaded');
            AJS.$('#upm-panel-install .upm-install-plugin-list').empty();
            parentContainer.show();
            if (options.type == 'search') {
                searchForPlugins(null, true, callback);
            } else {
                removeSearchOption();
                if (upm.resources[options.type]) {
                    container.removeData('pagination').find('div.upm-plugin-list-container').remove();
                    container.find('div.upm-development-product-version,div.upm-unknown-product-version').remove();
                    upm.filterPlugins.container = null;
                    AJS.$('p.upm-info', container).remove();
                    loadPlugins(upm.resources[options.type], options.category, options.cost, listOptions, container, parentContainer, callback);
                }
            }

            if (!skipPushState) {
                doPushState();
            }
        }

        function doPushState() {
            var options = getFilterOptions(),
                webUrl = constructMarketplaceUri(isUserFacing() ? upm.resources['plugin-requests'] : upm.resources['marketplace'], options);

            upm.pushState({
                type: options.type,
                cost: options.cost,
                category: options.category,
                q: options.q
            }, webUrl, webUrl);
        }

        function getFilterOptions() {
            var filterType = AJS.$('#upm-install-type'),
                filterCost = AJS.$('#upm-install-cost'),
                filterCategory = AJS.$('#upm-install-category');

            return {
                type: filterType.find('input').val(),
                category: filterCategory.find('input').val(),
                cost: filterCost.find('input').val(),
                q: AJS.$('#upm-install-search-box').val()
            };
        }

        function changeFilterState(options) {
            var filterCost = AJS.$('#upm-install-cost'),
                filterCategory = AJS.$('#upm-install-category');

            if (options.type === 'top-grossing') {
                disableFilter(filterCost, 'marketplace');
                enableFilter(filterCategory);
            } else if (options.type === 'search' || options.type === 'most-requested') {
                disableFilter(filterCost, '');
                disableFilter(filterCategory, '');
            } else {
                enableFilter(filterCost);
                enableFilter(filterCategory);
            }
        }

        function disableFilter(filter, value) {
            var currentValue = AJS.$(filter).find('input').val();

            $('ol li[data-value="' + value + '"]', filter).trigger('click', true);
            filter.addClass('disabled');

            if (currentValue !== value) {
                // save previous value for later use
                filter.attr('data-prev-value', currentValue);
                // don't make REST request, a new one will be triggered by the click above
                return false;
            }
        }

        function enableFilter(filter) {
            var previousValue = AJS.$(filter).attr('data-prev-value');

            if (!AJS.$(filter).hasClass('disabled')) {
                return;
            }

            filter.removeAttr('data-prev-value').removeClass('disabled');
            if (previousValue || previousValue === '') {
                // reinstate previous cost picker value, if available
                $('ol li[data-value="' + previousValue + '"]', filter).trigger('click', true);
                // don't make REST request, a new one will be triggered by previous line
                return false;
            }
        }

        function constructMarketplaceUri(baseUri, options) {
            // {baseUri}/{type}?category={category}&cost={cost}
            var url = baseUri || '',
                params = {},
                queryString;

            if (!options) {
                return url;
            }

            if (options.restPluginKey) {
                url += '/' + options.restPluginKey + "-key";
            } else if (options.pluginKey) {
                url += '/plugins/' + options.pluginKey;
            } else if (options.type) {
                url += '/' + options.type;
            }
            if (options.category) {
                params['category'] = options.category;
            }
            if (options.cost && options.type !== "top-grossing") {
                params['cost'] = options.cost;
            }
            if (options.type === 'search' && options.q) {
                params['q'] = options.q;
            }

            queryString = $.param(params, true);
            if (queryString) {
                url += '?' + queryString;
            }

            return url;
        }

        function initFilters() {
            var paths = window.location.pathname.split('/'),
                type = paths[paths.length - 1],
                cost = upm.urlParam('cost') || '',
                category = upm.urlParam('category') || '',
                q = upm.urlParam('q') || '',
                pluginKey;

            if (paths[paths.length - 2] === 'plugins') {
                type = 'single';
                pluginKey = paths[paths.length - 1];
            } else if (type === 'requests' || type === 'view') {
                type = 'most-requested';
            } else if (type === 'plugins') {
                type = 'recent';
            } else if (type !== 'search' && type !== 'purchases' && $('ol li[data-value="' + type + '"]', AJS.$('#upm-install-type')).size() == 0) {
                type = 'featured';
            }

            reloadFilters({
                type: type,
                category: category,
                cost: cost,
                q: q,
                pluginKey: pluginKey
            });
        }

        function reloadFilters(state) {
            var filterType = AJS.$('#upm-install-type'),
                filterCost = AJS.$('#upm-install-cost'),
                filterCategory = AJS.$('#upm-install-category'),
                searchBox = AJS.$('#upm-install-search-box');

            if (state.type === 'single' && state.pluginKey) {
                AJS.$('#carousel, #upm-install-search-form-container, .fancy-select').hide();
                selectPlugin(constructMarketplaceUri(upm.resources['available'], {restPluginKey : state.pluginKey}));
                return;
            }

            if (!isPurchasedAddonsPage()) {
                AJS.$('#carousel, #upm-install-search-form-container, .fancy-select').show();
            }
            if (state.type === 'search') {
                searchBox.val(state.q).trigger('focus');
                searchForPlugins(null, true);
            } else {
                filterType.find('input').val(state.type);
                filterCategory.find('input').val(state.category);
                filterCost.find('input').val(state.cost);
                $('ol li[data-value="' + state.type + '"]', filterType).trigger('click', true);
                $('ol li[data-value="' + state.category + '"]', filterCategory).trigger('click', true);
                $('ol li[data-value="' + state.cost + '"]', filterCost).trigger('click', true);
                changeDisplayedPlugins(true);
            }
        }

        /**
         * Creates and loads a list of installable plugins
         * @method loadInstallTab
         * @param {HTMLElement} container Container to load content into
         */
        function loadInstallTab(container) {
            if (AJS.params.isOnDemand) {
                AJS.$('#upm-install-search-form-container').addClass('hidden');
            }

            upm.bindOnce('pluginsLoaded.install', function(e, response) {
                container.addClass('loaded');
                upm.trigger('panelLoaded', [response]);
            });

            if (isViewRequestsPage()) {
                changeDisplayedPlugins(true);
            } else {
                loadCategoriesDropdown(function() {
                    initFilters();
                });
            }
        }

        function loadCategoriesDropdown(callback) {
        	if (AJS.$('#upm-install-category').length) {
	            var url = upm.resources['categories'];
	            if (url) {
	                AJS.$.ajax({
	                    url: url,
	                    type: 'get',
	                    dataType: 'json',
	                    success: function(response) {
	                        var categories = response;
	
	                        if (categories && categories.length) {
	                            for (var i = 0, l = categories.length; i < l; i++) {
	                                var li = AJS.$('<li></li>').attr('data-value', categories[i]).text(categories[i]);
	                                AJS.$('#upm-install-category .options').append(li);
	                            }
	                        }
	                        callback && callback();
	                    }
	                });
	            }
        	} else {
        		callback && callback();
        	}
        }

        function isUserFacing() {
            return (AJS.$('.upm-requests').length != 0) //only true when on page where requests are submitted
        }

        function isViewRequestsPage() {
            return (AJS.$('.upm-view-requests').length != 0); //only true when on page where requested are approved/rejected
        }

        function isPurchasedAddonsPage() {
            return (AJS.$('.upm-purchased-addons').length != 0);
        }


        /**
         * Display (un-hide) both the Staff Picked and Most Requested plugin lists.
         * This should be called on the initial page load after both lists have been
         * loaded but hidden, such that the end-user sees both appear at the same time regardless
         * of which list actually completes building first.
         */
        function displayInitialPlugins() {
            AJS.$('.upm-install-type.loaded').show();
        }

        function videoRef(youtubeId, autoplay) {
            return {
                height: 480,
                href : 'https://www.youtube.com/embed/' + youtubeId + '?autoplay=' + +autoplay + '&wmode=opaque', // +autoplay will convert a boolean to a number (0 or 1)
                type : 'iframe',
                width: 800
            };
        }

        function getYoutubeScreenshot(youtubeId) {
            return 'http://img.youtube.com/vi/' + youtubeId + '/0.jpg';
        }

        /**
         * Creates and returns an html element with plugin details for the "Install" tab
         * @method buildInstallDetails
         * @param {Object} plugin Object containing the plugin detail information
         * @return {HTMLElement} Plugin details element
         */
        function buildInstallDetails(plugin) {
            buildInstallDetails.template = buildInstallDetails.template || AJS.$(AJS.$('#upm-plugin-details-install').html());
            var details = buildInstallDetails.template.clone(),
                screenshotsContainer = AJS.$('div.upm-plugin-screenshots', details),
                videoContainer = AJS.$('div.upm-plugin-video', details),
                numScreenshots = plugin.versionDetails.screenshots.length || 0,
                pluginSystemVersionText;

            AJS.$('a.upm-plugin-details-link', details).html(upm.html_sanitize(plugin.name, upm.htmlSanitizerUrlPolicy));

            if (plugin.versionDetails && plugin.versionDetails.pluginSystemVersion) {
                pluginSystemVersionText = AJS.params['upm.plugin.system.' + plugin.versionDetails.pluginSystemVersion.toLowerCase()];
                if (pluginSystemVersionText) {
                    AJS.$('li.upm-plugin-system-version', details).text(pluginSystemVersionText);
                } else {
                    AJS.$('li.upm-plugin-system-version', details).remove();
                }
            } else {
                AJS.$('li.upm-plugin-system-version', details).remove();
            }
            if (plugin.license) {
                AJS.$('li.upm-plugin-license', details).text(AJS.format(AJS.params['upm.plugin.license.type'], plugin.license || AJS.params.upmTextUnknown));
            } else {
                AJS.$('li.upm-plugin-license', details).remove();
            }
            if (plugin.stable !== false) {
                AJS.$('li.upm-plugin-non-stable', details).remove();
            }

            AJS.$('li.upm-plugin-available-version', details).text('v. ' + plugin.version);

            if (plugin.description) {
                AJS.$('div.upm-plugin-description', details)
                    .html(upm.html_sanitize(plugin.description || AJS.params['upm.plugin.no.description'], upm.htmlSanitizerUrlPolicy))
                    .find('a').attr('target', '_blank');
            } else {
                AJS.$('div.upm-plugin-description', details).addClass('upm-no-description-text');
            }

            if ((plugin.installed || plugin.userInstalled) && plugin.links['plugin-review']) {
                AJS.$('.upm-plugin-review', details).replaceWith(upm.buildPluginReview(plugin));
                upm.checkUserReview(plugin, details);
            }

            if (plugin.links.binary) {
                AJS.$('input.upm-plugin-binary', details).val(plugin.links.binary);
            }

            if (plugin.links['plugin-watch']) {
                AJS.$('.upm-plugin-links', details).prepend(upm.buildPluginWatchLinks());
                upm.checkWatchStatus(plugin, details);
            }

            if (plugin.versionDetails.releaseNotesUrl) {
                AJS.$('a.upm-plugin-release-notes-link', details).attr('href', plugin.versionDetails.releaseNotesUrl);
            } else {
                AJS.$('a.upm-plugin-release-notes-link', details).closest('li').remove();
            }

            if (plugin.versionDetails.documentationUrl) {
                AJS.$('a.upm-plugin-documentation-link', details).attr('href', plugin.versionDetails.documentationUrl);
            } else {
                AJS.$('a.upm-plugin-documentation-link', details).closest('li').remove();
            }

            if (plugin.versionDetails.supportUrl) {
                AJS.$('a.upm-plugin-support-link', details).attr('href', plugin.versionDetails.supportUrl);
            } else {
                AJS.$('a.upm-plugin-support-link', details).closest('li').remove();
            }

            if (plugin.versionDetails.licenseUrl) {
                AJS.$('a.upm-plugin-license-link', details).attr('href', plugin.versionDetails.licenseUrl);
            } else {
                AJS.$('a.upm-plugin-license-link', details).closest('li').remove();
            }

            if (plugin.versionDetails.reviewsUrl) {
                AJS.$('a.upm-plugin-reviews-link', details).attr('href', plugin.versionDetails.reviewsUrl);
            } else {
                AJS.$('a.upm-plugin-reviews-link', details).closest('li').remove();
            }

            if (plugin.versionDetails.pricingUrl) {
                AJS.$('a.upm-plugin-pricing-link', details).attr('href', plugin.versionDetails.pricingUrl);
            } else {
                AJS.$('a.upm-plugin-pricing-link', details).closest('li').remove();
            }

            if (plugin.links.details) {
                AJS.$('a.upm-plugin-marketplace-link', details).attr('href', plugin.links.details);
                AJS.$('input.upm-plugin-homepage', details).val(plugin.links.details);
            } else {
                AJS.$('a.upm-plugin-marketplace-link', details).closest('li').remove();
            }

            if (numScreenshots) {
                AJS.$('img.upm-plugin-top-screenshot', screenshotsContainer).attr('src', plugin.versionDetails.screenshots[0].link);
                if (plugin.versionDetails.screenshots[0].width < plugin.versionDetails.screenshots[0].height) {
                    AJS.$('img.upm-plugin-top-screenshot', screenshotsContainer)
                        .removeAttr('width')
                        .attr('height', 185);
                }
                AJS.$('a', screenshotsContainer).click(function (e) {
                    e.preventDefault();
                    AJS.$.fancybox(
                        upm.screenshotRefs(plugin.versionDetails.screenshots),
                        {
                            changeSpeed: 0,
                            cyclic: true,
                            overlayColor: '#000',
                            overlayOpacity: 0.8,
                            speedIn: 0,
                            speedOut: 0,
                            transitionIn: 'none',
                            transitionOut: 'none'
                        });
                });
                AJS.$('span.upm-plugin-screenshots-label', screenshotsContainer).text(
                    AJS.format(
                        numScreenshots == 1 ? AJS.params['upm.plugin.details.screenshot'] : AJS.params['upm.plugin.details.screenshots'],
                        numScreenshots));
            } else {
                screenshotsContainer.remove();
            }

            if (plugin.versionDetails.youtubeId) {
                AJS.$('img.upm-plugin-video-screenshot', videoContainer).attr('src', getYoutubeScreenshot(plugin.versionDetails.youtubeId));
                AJS.$('a', videoContainer).click(function (e) {
                    e.preventDefault();
                    AJS.$.fancybox(
                        videoRef(plugin.versionDetails.youtubeId, true),
                        {
                            changeSpeed: 0,
                            cyclic: true,
                            overlayColor: '#000',
                            overlayOpacity: 0.8,
                            speedIn: 0,
                            speedOut: 0,
                            transitionIn: 'none',
                            transitionOut: 'none'
                        });
                });
            } else {
                videoContainer.remove();
            }

            AJS.$('.upm-plugin-actions', details).replaceWith(buildPluginActionButtons(plugin));

            AJS.$('div.upm-plugin-actions .toolbar-dropdown', details).dropDown('Standard', {alignment: 'right'});
            
            buildPluginRecommendations(plugin.links['recommendations'], details);

            if (getFilterOptions().type === 'most-requested' && plugin.requests && plugin.requests.length) {
                buildPluginRequests(plugin, details);
            }

            return details;
        }

        /**
         * Searches PAC for plugins that match the entered text
         * @method searchForPlugins
         * @param {Event} e The event object
         */
        function searchForPlugins(e, skipPushState, callback) {
            var form = AJS.$('#upm-install-search-form'),
                input = form.find('input'),
                term = input.val(),
                container = AJS.$('#upm-install-search'),
                parentContainer = AJS.$('#upm-install-container-search'),
                filterType = AJS.$('#upm-install-type');

            e && e.preventDefault();
            if (term && term != AJS.params.upmTextInstallSearchBox) {
                // if some filter option other than Search is currently selected, add the extra Search item
                // and select it; this will trigger an event that will end up coming back to searchForPlugins
                // and then we'll take the other code path below.
                if (filterType.find('input').val() != 'search') {
                    AJS.$('<li data-value="search" id="upm-search-option"></li>')
                            .text(AJS.params['upmTextSearchOption'])
                            .prependTo(filterType.find('ol'));
                    $('#upm-search-option').trigger('click', skipPushState);
                } else {
                    AJS.$('#upm-panel-install .upm-install-type').hide();
                    container.empty()
                        .removeData('pagination')
                        .append(AJS.$('<h3></h3>').text(AJS.format(AJS.params.upmTextSearchResults, term)));
                    parentContainer.show();

                    if (!skipPushState) {
                        doPushState();
                    }

                    loadPlugins(upm.resources['recent'] + '?' +  form.serialize(), '', '', listOptions, container, parentContainer, callback);
                }
            } else {
                //return to the "featured" list if you submit an empty search form
                filterType.find('li.upm-filter-featured').click();
            }
        }

        function loadBannerCarousel() {
        	if (AJS.$('#carousel').length) {
	            var url = upm.resources['banners'];

	            if (url) {
	                AJS.$.ajax({
	                    url: url,
	                    type: 'get',
	                    dataType: 'json',
	                    success: function(response) {
	                        // Only load banners if we have at least 3
	                        if (response && response.count && response.count > 3) {
	                            initCarousel(response);
	                        }
	                    }
	                });
	            }
        	}
        }

        function newCarouselItem(banner) {
            var item = AJS.$('<li class="carousel-item-wrap"></li>')
                    .attr('data-expand', banner.pluginKey)
                    .attr('data-rest-uri', banner.links.self),
                link = AJS.$('<a href="" class="carousel-item"></a> '),
                image = AJS.$('<img>').attr('src', banner.image.links['binary']).attr('alt', banner.image.altText);
            return item.append(link.append(image));
        }

        function loadNewBanners(prev, carousel) {
            var that = this,
                currentTail = that.infinity.tail,
                url = that.infinity.next;

            if (prev) {
                url = that.infinity.prev;
            }

            AJS.$.ajax({
                url: url,
                type: 'get',
                dataType: 'json',
                success: function(response) {
                    for (var i = 0; banner = response.banners[i++];) {
                        if (banner.image && banner.image.links['binary']) {
                            carousel.insertAfter(currentTail + i - 1, AJS.$(newCarouselItem(banner)));
                            if (prev) {
                                that.infinity.head = that.infinity.tail + 1;
                            } else {
                                that.infinity.tail = that.infinity.tail + 1;
                            }
                        }
                    }
                    if (prev) {
                        that.infinity.prev = response.links.prev;
                    } else {
                        that.infinity.next = response.links.next;
                    }
                }
            });
        }

        function initCarousel(rep) {
            var carousel = AJS.$('<ul class="cloud-carousel"></ul>'),
                carouselContainer = AJS.$('#carousel'),
                firstSlider,
                that = this,
                bannersCount = rep.banners.length;

            that.infinity = {
                head: 0,
                tail: 0,
                prev: rep.links.prev,
                next: rep.links.next,
                total: rep.count
            };

            for (var i = 0; banner = rep.banners[i++];) {
                if (banner.image && banner.image.links['binary']) {
                    carousel.append(newCarouselItem(banner));
                }
            }

            that.infinity.tail = AJS.$('.carousel-item-wrap', carousel).size() - 1;
            carousel.appendTo(carouselContainer);

            // Then initialize the carousel
            firstSlider = carousel.cloudCarousel({
                // start at a random place within the first page of the banners
                startIndex: Math.floor(Math.min(Math.max(2, Math.random() * bannersCount), bannersCount - 2)),
                autorotate: true,
                // On clicking the center item, expand that plugin
                click: function(e, slider) {
                    var position = slider.position;

                    e && e.preventDefault();

                    if (position === 'left' || position === 'right') {
                        if (AJS.$('.carousel-item-wrap', carousel).size() < that.infinity.total) {
                            if (AJS.$('.carousel-item-wrap:eq(' + that.infinity.head + ')').hasClass('left')) {
                                loadNewBanners(true, carousel);
                            } else if (AJS.$('.carousel-item-wrap:eq(' + that.infinity.tail + ')').hasClass('right')) {
                                loadNewBanners(false, carousel);
                            }
                        }
                    } else if (position === 'center') {
                        var url = slider.$container.attr('data-rest-uri'),
                            pluginKey = slider.$container.attr('data-expand'),
                            webUrl;

                        AJS.$('#single-container-back-link').data('fromPluginSelection', true);

                        selectPlugin(url);

                        webUrl = constructMarketplaceUri(isUserFacing() ? upm.resources['plugin-requests'] : upm.resources['marketplace'], {pluginKey : pluginKey});
                        upm.pushState({type: 'single',
                                       pluginKey: pluginKey}, webUrl, webUrl);
                    }
                }
                // When all images are done loading, remove the loading spinner
            }).bind('loaded', function() {
                firstSlider.removeClass('loading');

                // Add the loading spinner initially
            }).getSlider(0).$container.find('a').addClass('loading');
        }

        upm.setDetailsFn('install', buildInstallDetails);

        // Functionality shared by the update check and plugins page

        // hover and click functions for plugin lists
        $plugins = $('div.upm-plugin-list.expandable div.upm-plugin', upm.$container);
        if (isIE7) {
            $plugins.live('mouseover', function() {
                $(this).addClass('hover');
            });
            $plugins.live('mouseout', function() {
                $(this).removeClass('hover');
            });
        }
        // workaround for UPM-2549
        $plugins.find('div.upm-plugin-row').die('click.plugin-row').live('click.plugin-row', upm.pluginRowClick);

        $plugins.find('span.already-installed-message').live('click', function(e) {
            AJS.$(e.target).closest('div.upm-plugin-row').click();
        });

        // If toolbar buttons have disabled class, we want to kill the event
        $('a.toolbar-trigger', upm.$container).live('click', upm.checkButtonDisabledState);

        // UPM-930: IE lets you click on buttons even if they're disabled
        if (isIE) {
            $('button', upm.$container).live('click', upm.checkButtonDisabledState);
        }

        // plugin download button
        $('a.upm-download', upm.$container).live('click', upm.showDownloadDialog);

        // requires restart details link
        $('#upm-requires-restart-show').live('click', function (e) {
            e.preventDefault();
            e.target.blur();
            upm.toggleRequiresRestartDetails($(e.target));
        });

        // requires restart action cancellation link
        $('a.upm-requires-restart-cancel', upm.$messageContainer).live('click', upm.cancelActionRequiringRestart);

        // Create the carousel unless we are on the View Requests page
        if (!isViewRequestsPage()) {
            loadBannerCarousel();
        }

        upm.bindOnce('pluginsLoaded.install', function(e, response) {
            if (!isUserFacing() || mostRequestedLoaded) {
                displayInitialPlugins();
            }
        });

        function updateInstalledPlugin(pluginElement, pluginInfo) {
            var actionButtons = buildPluginActionButtons(pluginInfo);
            pluginElement.find('.upm-plugin-actions').each(function(index, el) {
                AJS.$(el).replaceWith(actionButtons.clone());
            });

            upm.fetchPacPluginDetails(pluginInfo, function(pacResponse) {
                if (pacResponse.links['plugin-review']) {
                    AJS.$('.upm-plugin-review', pluginElement).replaceWith(upm.buildPluginReview(pacResponse));
                    upm.checkUserReview(pacResponse, pluginElement);
                }
            });
        }

        // search box -- install tab
        upm.clearOnBlur('#upm-install-search-box', AJS.params.upmTextInstallSearchBox);

        // plugin install button, buy and try buttons (which also install)
        $('a.upm-install, a.upm-buy, a.upm-try', upm.$container).live('click', function(e) {

            var buttonTarget = AJS.$(e.target);
            if (buttonTarget.hasClass('upm-buy')) {
                upm.logAnalytics('buy', buttonTarget.closest('.upm-plugin'));
            } else if (buttonTarget.hasClass('upm-try')) {
                upm.logAnalytics('try', buttonTarget.closest('.upm-plugin'));
            }

            AJS.$(e.target).closest('.upm-plugin').bind('pluginInstalled', function(e, pluginInfo) {
                var target = AJS.$(e.target);
                e.preventDefault();
                target.unbind('pluginInstalled');
                updateInstalledPlugin(target.closest('.upm-plugin'), pluginInfo);
            });
            upm.installPlugin(e);
        });

        // pac search box
        $('#upm-install-search-form', upm.$container).submit(searchForPlugins);

        // exit safe mode link
        $('#upm-safe-mode-restore').click(function(e) {
            e.preventDefault();
            upm.showConfirmDialog(AJS.params.upmTextSafeModeRestoreConfirm, upm.restoreFromSafeMode);
        });

        // exit safe mode link
        $('#upm-safe-mode-keep-state').click(function(e) {
            e.preventDefault();
            upm.showConfirmDialog(AJS.params.upmTextSafeModeExitConfirm, upm.exitSafeMode);
        });

        // single plugin view back link
        $('#single-container-back-link').click(function(e) {
            var fromPluginSelection = AJS.$(this).data('fromPluginSelection') || false;

            e && e.preventDefault();

            if (fromPluginSelection) {
                window.history.back();
            } else {
                if(isIE8AndLower) {
                    window.location.href = upm.resources['marketplace'];
                } else {
                    reloadFilters({type : 'featured'});
                }
                
                doPushState();
            }
        });

        // Add-on review buttons
        AJS.$('.upm-review').live('click', upm.submitReview);

        AJS.$('.review-form span.star')
            .live('mouseover', upm.showRating)
            .live('mouseout', upm.resetRating)
            .live('click', upm.setRating);

        // make back button work in browsers that support 'onhashchange'
        $(window).bind('hashchange', function() {
            var hash = upm.getLocationHash(),
                    tab = hash.tab;
            if ($('#upm-panel-' + tab).is(':hidden')) {
                upm.loadTab(tab, hash.key);
            } else if (hash.key) {
                upm.focusPlugin(hash.key, tab);
            }
        });

        upm.bindPopState(function(e) {
            var state = e.state || (e.originalEvent && e.originalEvent.state);

            if (state) {
                reloadFilters(state);
            } else if (window.location.href === (isUserFacing() ? upm.resources['plugin-requests'] : upm.resources['marketplace'])) {
                reloadFilters({type : 'featured'});
            } else if (!isViewRequestsPage()) {
                initFilters();
            }
        });

        $('#upm-panel-install .fancy-select').bind('change', onFilterChange);

        upm.load('install', hash.key, hash.message);
        loadInstallTab(AJS.$('#upm-panel-install'));
        upm.checkPacAvailable();

        /* moving request elements here now that they are part of the find page*/

        function submitPluginRequest(pluginKey, pluginName, message) {
            AJS.$.ajax({
                type: 'POST',
                url: upm.resources['create-requests'],
                dataType: 'json',
                contentType: upm.contentTypes['upm'],
                data: upm.json.stringify({ "pluginKey": pluginKey, "pluginName": pluginName, "message": message }),
                // can't access the request object from the success fn (before jquery 1.4), so we have to use the 'complete' callback
                complete: function(request, status) {
                    var response = upm.json.parse(request.responseText),
                        success = (status == 'success'),
                        messageType = success ? 'success' : 'error',

                        message = AJS.params['upm.messages.request.' + (success ? 'success' : 'failure')];

                    showRequestMoreAddonsDialog(message, messageType, pluginKey);
                }
            });
        }

        function showRequestMoreAddonsDialog(message, messageType, success) {

            AJS.$.ajax({
                url: upm.resources['most-requested'],
                type: 'get',
                cache: false,
                dataType: 'json',
                data: { 'start-index': 0, 'max-results': 3, 'exclude-user-requests': true }, //only show a max of 3 most requested plugins, always starting at 0
                success: function(response) {
                    showRequestMoreAddonsDialog.template = showRequestMoreAddonsDialog.template || AJS.$(AJS.$('#upm-most-requested-dialog-template').html());

                    var plugins = response.plugins,
                        listOptions = {isExpandable: false, isInstalledList: false, className: 'install', showRequestCount: true},
                        dialogBody = AJS.$(showRequestMoreAddonsDialog.template.clone()),
                        pluginKey = success ? response.pluginKey : response.subCode,
                        mostRequestedContainer = dialogBody.find('.upm-dialog-most-requested-addons'),
                        pluginElement = upm.$container.find('input.upm-plugin-key[value="' + pluginKey + '"]').closest('div.upm-plugin'),
                        pluginDetails = pluginElement.find('div.upm-details');

                    if(plugins.length > 0) {
                        upm.displayMessage(dialogBody.find('div.upm-dialog-most-requested-addons'), {
                            body: message,
                            type: messageType ,
                            closeable: false
                        });

                        if (plugins && plugins.length) {
                            buildListOfRequestMoreAddons(mostRequestedContainer, plugins, listOptions)
                        }

                        mostRequestedLoaded = true;

                        var closeFn = function (dialog) {
                                dialog.remove();
                            },
                            popup = upm.createDialog({
                                width: 600,
                                height: 500,
                                id: 'upm-others-requested-dialog',
                                closeable: true,
                                closeFn: closeFn
                            });
                        popup.addHeader(AJS.params["upm.messages.request.most.title"]);
                        popup.addPanel("All", mostRequestedContainer);
                        popup.addButton(AJS.params.upmTextClose, closeFn);

                        $('#upm-others-requested-dialog').find('.upm-plugin-name, .category-link').click(function(){popup.remove()});
                        $('#upm-others-requested-dialog').find('a.upm-request').click(function(e) {
                            e.preventDefault();
                            showRequestSubmitDialog(e);
                            popup.remove();
                        });

                        popup.show();

                    } else {
                        upm.showInfoDialog(AJS.params["upm.messages.request.success.title"], '<p>' + AJS.params["upm.messages.request.success"] + '</p>')
                    }

                },
                error: function(request) {
                    upm.handleAjaxError(upm.$messageContainer, request, '');
                }
            });
        }

        function buildListOfRequestMoreAddons(container, plugins, options) {
            if ((plugins && plugins.length) || options.forceCreation) {
                for (var plugin in plugins){
                    container.append(buildRequestMoreAddonRow(plugins[plugin], options));
                }
            }
            return container;
        }

        function buildRequestMoreAddonRow(plugin, options) {
            buildRequestMoreAddonRow.template = buildRequestMoreAddonRow.template ||
                AJS.$(AJS.$('#upm-plugin-abbreviated-template').html());

            var addonRow = populateRowTemplate(plugin, buildRequestMoreAddonRow.template.clone(), options),
                addonUrl = constructMarketplaceUri(upm.resources['plugin-requests'], {pluginKey : plugin.key});

            AJS.$('.upm-plugin-name-link', addonRow).attr('href', addonUrl);

            return addonRow;
        }

        function showRequestSubmitDialog(e) {
            showRequestSubmitDialog.template = showRequestSubmitDialog.template || AJS.$(AJS.$('#upm-request-plugin-dialog-template').html());

            var dialogBody = AJS.$(showRequestSubmitDialog.template.clone()),
                pluginElement = AJS.$(e.target).closest('div.upm-plugin'),
                existingMessage = pluginElement.find('.upm-request-existing-message'),
                isUpdate = existingMessage.length != 0,
                dialogHeading = AJS.params['upm.messages.request.' + (isUpdate ? 'update.' : '') + 'heading'],
                buttonText = AJS.params['upm.messages.request.' + (isUpdate ? 'update.' : '') + 'button'],
                buttonHandler = function(dialog) {
                    var pluginKey = AJS.$('#pluginRequestPluginKey').val(),
                        pluginName = AJS.$('#pluginRequestPluginName').val(),
                        message = AJS.$('#pluginRequestMessage').val(),
                        pluginElement = upm.$container
                            .find('input.upm-plugin-key[value="' + pluginKey + '"]')
                            .closest('div.upm-plugin'),
                        previousMessage = pluginElement.find('.upm-request-existing-message');

                    pluginElement.find('a.upm-request').text(AJS.params['upm.plugin.action.request.update']);
                    if (previousMessage.length) {
                        pluginElement.find('.upm-request-existing-message').attr('value', message);
                    } else {
                        AJS.$('<input>')
                            .attr('type', 'hidden')
                            .attr('value', message || '')
                            .attr('class', 'upm-request-existing-message')
                            .appendTo(pluginElement.find('.upm-plugin-actions'));
                    }

                    submitPluginRequest(pluginKey, pluginName, message);
                    dialog.remove();
                };

            if (isUpdate) {
                dialogBody.find('.upm-dialog-new-request').addClass('hidden');
                dialogBody.find('.upm-dialog-update-request').removeClass('hidden');
                dialogBody.find('#pluginRequestMessage')
                    .text(existingMessage.val())  //display text
                    .val(existingMessage.val());  //actual value
            }

            dialogBody.find('#pluginRequestPluginKey').val(pluginElement.find('.upm-plugin-key').val());
            dialogBody.find('#pluginRequestPluginName').val(pluginElement.find('.upm-plugin-name').text());

            upm.showPluginStatusDialog(dialogHeading, pluginElement, null, buttonText, buttonHandler, dialogBody.html());
        }

        if(!isUserFacing()) {
            AJS.$('#upm-link-bar').removeClass('hidden');
        }

        if (!upm.resources['most-requested']) {
            AJS.$('.upm-filter-requested').remove();
            AJS.$('#upm-install-type .options li').last().addClass('last');
            if (getFilterOptions().type == 'most-requested') {
                reloadFilters({type : 'featured'});
            }
        }
    };

    $(window).bind('upmready', upmReady);
})(AJS.$);
