/*
 * This file drives the osgi page
 */
(function($) {

    // Wait for UPM to load
    var upmReady = function() {

        // Stop waiting for UPM to load
        $(window).unbind('upmready', upmReady);

        var $plugins,
            modules,
            isIE = upm.browser.isIE,
            isIE7 = upm.browser.isIE7,
            // Find out which page we are on
            isUpdateCheck = !!$('#upm-panel-compatibility').length,
            hash = upm.getLocationHash('manage');

        // Functionality shared by the update check and plugins page

        // hover and click functions for plugin lists
        $plugins = $('div.upm-plugin-list.expandable div.upm-plugin', upm.$container);
        if (isIE7) {
            $plugins.live('mouseover', function() {
                $(this).addClass('hover');
            });
            $plugins.live('mouseout', function() {
                $(this).removeClass('hover');
            });
        }
        // workaround for UPM-2549
        $plugins.find('div.upm-plugin-row').die('click.plugin-row').live('click.plugin-row', upm.pluginRowClick);

        // hover and click functions for plugin modules
        modules = $('div.upm-module', upm.$container);
        if (isIE7) {
            modules.live('mouseover', function() {
                $(this).addClass('hover');
            });
            modules.live('mouseout', function() {
                $(this).removeClass('hover');
            });
        }

        // Toggle viewing of plugin modules
        $('a.upm-module-toggle', upm.$container).live('click', function(e) {
            var target = $(e.target).blur();
            e.preventDefault();
            target.closest('div.upm-plugin-modules').toggleClass('expanded');
        });

        // If toolbar buttons have disabled class, we want to kill the event
        $('a.toolbar-trigger', upm.$container).live('click', upm.checkButtonDisabledState);

        // UPM-930: IE lets you click on buttons even if they're disabled
        if (isIE) {
            $('button', upm.$container).live('click', upm.checkButtonDisabledState);
        }

        // plugin disable button
        $('a.upm-disable', upm.$container).live('click', upm.disablePlugin);

        // plugin enable button
        $('a.upm-enable', upm.$container).live('click', upm.enablePlugin);

        // module disable button
        $('a.upm-module-disable', upm.$container).live('click', upm.disableModule);

        // module enable button
        $('a.upm-module-enable', upm.$container).live('click', upm.enableModule);

        // enable safe mode link
        $('#upm-safe-mode-enable').click(function(e) {
            e.preventDefault();
            upm.showConfirmDialog(AJS.params.upmTextSafeModeConfirm, upm.enableSafeMode);
        });

        // exit safe mode link
        $('#upm-safe-mode-restore').click(function(e) {
            e.preventDefault();
            upm.showConfirmDialog(AJS.params.upmTextSafeModeRestoreConfirm, upm.restoreFromSafeMode);
        });

        // exit safe mode link
        $('#upm-safe-mode-keep-state').click(function(e) {
            e.preventDefault();
            upm.showConfirmDialog(AJS.params.upmTextSafeModeExitConfirm, upm.exitSafeMode);
        });

        // requires restart details link
        $('#upm-requires-restart-show').live('click', function (e) {
            e.preventDefault();
            e.target.blur();
            upm.toggleRequiresRestartDetails($(e.target));
        });

        // requires restart action cancellation link
        $('a.upm-requires-restart-cancel', upm.$container).live('click', upm.cancelActionRequiringRestart);

        // search box -- osgi tab
        upm.clearOnBlur('#upm-osgi-search-box', AJS.params.upmTextOsgiSearchBox);

        // filter box -- osgi tab
        upm.clearOnBlur('#upm-osgi-filter-box', AJS.params.upmTextFilterOsgiBundles);

        // osgi bundle search box
        $('#upm-osgi-search-form', upm.$container).submit(upm.filterBundles);

        // Banner click to update the UPM by secretly doing what a user would do if the UPM row were visible. The UPM row is not in the DOM
        // but it cached, so we are going to fake clicks on it to expand it, then click the update button. This is currently the path of
        // least resistence to make UPM updatable by a banner, since the update code path is reliant on clicking on the update button inline.
        $('a#upm-banner-update', upm.$container).live('click', function(e) {
            var pluginElement = upm.upmPluginContainer.closest('div.upm-plugin'),
                details = upm.upmPluginContainer.find('div.upm-details');
            e.preventDefault();

            upm.startProgress(AJS.params.upmTextSelfUpdateWorking);

            // Hack to close the banner
            $(e.target).closest('div.aui-message').remove();

            pluginElement.bind('pluginLoaded.update', function() {
                upm.updatePlugin({
                    target: pluginElement
                });
            });

            // The URL to talk to doesn't exist until the UPM row (a hidden element) is expanded, so expand it
            upm.togglePluginDetails(pluginElement);
        });

        // osgi bundle cross-references
        $('a.upm-osgi-bundle-xref').live('click', function (e) {
            e.preventDefault();
            e.target.blur();
            upm.focusPlugin($(e.target).next('input').attr('value'), 'osgi');
        });

        // make back button work in browsers that support 'onhashchange'
        $(window).bind('hashchange', function() {
            var hash = upm.getLocationHash(),
                    tab = hash.tab;
            if ($('#upm-panel-' + tab).is(':hidden')) {
                upm.loadTab(tab, hash.key);
            } else if (hash.key) {
                upm.focusPlugin(hash.key, tab);
            }
        });

        upm.loadTab('osgi', hash.key, hash.message);
        upm.checkPacAvailable();
    };

    $(window).bind('upmready', upmReady);
})(AJS.$);
