/*
* This file drives the Purchased Add-ons page
*/
(function($) {

    // Wait for UPM to load
    var upmReadyPurchasedAddons = function() {

        // Stop waiting for UPM to load
        $(window).unbind('upmready', upmReadyPurchasedAddons);
        
        $('#upm-check-available-licenses').click(openUpdateLicensesDialog);            
        
        upm.bind('pluginsLoaded.install', function(e, response) {
            var incompatibles = response.incompatiblePlugins,
                unknowns = response.unknownPlugins,
                hasIncompatible = !(!incompatibles || !incompatibles.length),
                hasUnknown = !(!unknowns || !unknowns.length),
                incompatibleDiv = $('#upm-purchases-incompatible-plugins'),
                mpacAvailable = response.hostStatus && !(response.hostStatus.pacDisabled || response.hostStatus.pacUnavailable),
                canCheck = mpacAvailable && upm.resources['update-licenses'], 
                unknownDiv = mpacAvailable ? $('#upm-purchases-unknown-plugin-keys') :
                                             $('#upm-purchases-unknown-plugin-keys-no-mpac');
                                
            $('#upm-purchased-addons-header-default').toggleClass('hidden', !mpacAvailable);

            if (hasIncompatible) {
                var listElem = $('<ul></ul>');
                for (var i = 0; i < incompatibles.length; i++) {
                    var p = incompatibles[i],
                        name = p.name,
                        mpacUrl = p.links && p.links.details,
                        itemElem = $('<li></li>').append($('<a></a>').text(name).attr('href', mpacUrl));
                    listElem.append(itemElem);
                }
                incompatibleDiv.find('ul').replaceWith(listElem);
            }
            incompatibleDiv.toggleClass('hidden', !hasIncompatible);
            if (hasUnknown) {
                var listElem = $('<ul></ul>');
                for (var i = 0; i < unknowns.length; i++) {
                    var p = unknowns[i],
                        title = p.name || p.key,
                        link = p.links && p.links['plugin-details'],
                        listItem = $('<li></li>');
                    if (link) {
                        listItem.append($('<a></a>').text(title).attr('href', link));
                    } else {
                        listItem.text(title);
                    }
                    listElem.append(listItem);
                }
                unknownDiv.find('ul').replaceWith(listElem);
            }
            unknownDiv.toggleClass('hidden', !hasUnknown);
        });
    };

    function openUpdateLicensesDialog() {
        var popup = upm.createDialog(600, 288, 'upm-update-licenses-dialog'),
            $dialog,
            $usernameEdit,
            $passwordEdit,
            $submitBtn,
            $errorField,
            submitFn;

        popup.addPanel('All', AJS.$('#upm-update-licenses-dialog-template').html());
        $dialog = AJS.$('#upm-update-licenses-dialog');
        $submitBtn = AJS.$('.login', $dialog);
        $usernameEdit = AJS.$('#upm-update-licenses-username');
        $passwordEdit = AJS.$('#upm-update-licenses-password');
        $errorField = AJS.$('.request-error', $dialog);
        
        submitFn = function() {
            var username = AJS.$.trim($usernameEdit.val()),
                password = AJS.$.trim($passwordEdit.val()),
                $throbber = AJS.$('<div class="loading"></div>');

            $errorField.hide();

            if (!(username && password)) {
                AJS.$('.username', $dialog).toggleClass('empty', !username);
                AJS.$('.password', $dialog).toggleClass('empty', !password);
                return;
            }
            AJS.$('.field', $dialog).removeClass('empty');
            $submitBtn.attr('disabled', 'disabled');
            AJS.$('.buttons', $dialog).append($throbber);
            updateLicenses(username, password,
                function() {
                    popup.remove();
                },
                function(error) {
                    $throbber.remove();
                    $submitBtn.removeAttr('disabled');
                    $errorField.html(AJS.params[error] || AJS.params['upm.purchased.addons.check.error.unknown']).show();
                });
        };
        $submitBtn.click(submitFn);
        AJS.$('input', $dialog).keypress(function(e) {
            if (e.which === 13) { // Return key
                submitFn();
                return false;
            }
        });
        AJS.$('.cancel', $dialog).click(function() {
            popup.remove();
        })
        
        popup.show();
        AJS.$('.dialog-panel-body', $dialog).css('height', ''); // AUI display bug
        
        $usernameEdit.focus();
    }
    
    function updateLicenses(username, password, successCallback, errorCallback, canRetry) {
        var url = upm.resources['update-licenses'] + '?token=' + AJS.$('#upm-install-token').val(),
            credentials = {'username': username, 'password': password},
            currentCount = AJS.$('#upm-install-purchases .upm-plugin').length,
            updatedPluginKeys;
        
        AJS.$('#upm-purchases-none, #upm-purchases-none-new').addClass('hidden');
        AJS.$.ajax({
            url: url,
            type: 'post',
            contentType: upm.contentTypes.upm,
            dataType: 'json',
            data: upm.json.stringify(credentials),
            complete: function(request, status) {
                var response = upm.json.parse(request.responseText);
                if (status == 'success') {
                    if (response.error) {
                        errorCallback && errorCallback(response.error);
                    } else {
                        updatedPluginKeys = response.updatedLicensePluginKeys;
                        if (updatedPluginKeys && updatedPluginKeys.length) {
                            upm.reloadPlugins(function() {
                                for (var i = 0; i < updatedPluginKeys.length; i++) {
                                    AJS.$('#upm-plugin-' + upm.createHash(updatedPluginKeys[i], 'install')).addClass('new-license');
                                }
                            });
                        } else {
                            AJS.$(currentCount ? '#upm-purchases-none-new' : '#upm-purchases-none').removeClass('hidden');
                            AJS.$('#upm-install-purchases .upm-plugin').removeClass('new-license');
                        }
                        successCallback && successCallback();
                    }
                    upm.getAndStoreAntiXsrfToken();
                } else {
                    if (!upm.reloadIfWebSudoError(response.subCode)) {
                        // try again, with new token, exactly once -- one retry per user request
                        if (response.subCode == 'upm.error.invalid.token' && canRetry) {
                            upm.getAndStoreAntiXsrfToken(function() {
                                updateLicenses(username, password, successCallback, errorCallback, true);
                            });
                        } else {
                            errorCallback && errorCallback(AJS.params['upm.purchased.addons.check.error.unknown']);
                            upm.getAndStoreAntiXsrfToken();
                        }
                    }
                }
            }
        });
    }

    $(window).bind('upmready', upmReadyPurchasedAddons);
})(AJS.$);
