AJS.test.require('com.atlassian.bamboo.plugins.atlassian-bamboo-jenkins-importer:jenkinsImporterWeb');

module('BAMBOO.JENKINS_IMPORTER.Progressbar', {
    setup: function () {
        var that = this;
        this.xhr = sinon.useFakeXMLHttpRequest();
        this.requests = [];
        this.xhr.onCreate = function (req) { that.requests.push(req); };
        this.clock = sinon.useFakeTimers();
    },
    teardown: function () {
        this.xhr.restore();
        this.clock.restore();
    }
});

test('#getProgress', function () {
    var pb = new BAMBOO.JENKINS_IMPORTER.Progressbar({});
    var responseHeaders = { 'Content-Type': 'application/json' };
    var responseBody = JSON.stringify({ chainsImported: 0, chainsToImport: 1, importFinished: false });

    pb.updateProgressbar = sinon.spy();
    pb.scheduleUpdate = sinon.spy();
    pb.redirectOnCompletion = sinon.spy();

    this.requests[0].respond(200, responseHeaders, responseBody);
    equal(this.requests.length, 1, 'is called on initialisation');

    pb.getProgress();
    this.requests[1].respond(200, responseHeaders, responseBody);
    equal(pb.updateProgressbar.callCount, 1, 'calls #updateProgressbar after server responds successfully');
    equal(pb.redirectOnCompletion.callCount, 1, 'calls #redirectOnCompletion after server responds successfully');
    equal(pb.scheduleUpdate.callCount, 1, 'calls #scheduleUpdate after server responds successfully');

    pb.getProgress();
    this.requests[2].respond(404, responseHeaders, '');
    equal(pb.updateProgressbar.callCount, 1, 'doesn\'t call #updateProgressbar after server fails to respond');
    equal(pb.redirectOnCompletion.callCount, 1, 'doesn\'t call #redirectOnCompletion after server fails to respond');
    equal(pb.scheduleUpdate.callCount, 2, 'calls #scheduleUpdate after server fails to respond');
});

test('#scheduleUpdate', function () {
    var pb = new BAMBOO.JENKINS_IMPORTER.Progressbar({
        timeout: 100
    });

    pb.getProgress = sinon.spy();

    pb.scheduleUpdate();
    pb.scheduleUpdate();
    this.clock.tick(150);
    equal(pb.getProgress.callCount, 1, 'timeout is cleared so that #getProgress is only called once');
});


module('BAMBOO.JENKINS_IMPORTER.JobSelector', {
    setup: function () {
        this.$fixture = jQuery('#qunit-fixture');
        this.$div1 = jQuery('<div id="foo"><input type="checkbox" class="selected" checked /></div>').appendTo(this.$fixture);
        this.$div2 = jQuery('<div id="bar"><input type="checkbox" class="selected" /></div>').appendTo(this.$fixture);
        this.jobSelector = new BAMBOO.JENKINS_IMPORTER.JobSelector({
            classes: {
                table: 'my-table',
                select: 'selected'
            }
        });
    },
    teardown: function () {
    }
});

test('#selectToggle', function () {
    this.jobSelector.selectToggle = sinon.spy();
    var $myTable = jQuery('<table class="my-table"><tr><td><input type="checkbox" class="selected" /></td></tr></table>').appendTo(this.$fixture);
    this.jobSelector.init();

    $myTable.find('input[type="checkbox"]').change();
    ok(this.jobSelector.selectToggle.called, 'is called after change event is fired on descendant');
});

test('#toggleFields', function () {
    var $checkbox1 = jQuery('<input type="checkbox" id="foo" />').appendTo(this.$fixture);
    var $checkbox2 = jQuery('<input type="checkbox" id="bar" disabled />').appendTo(this.$fixture);

    this.jobSelector.toggleFields(this.$fixture, '#foo', true);
    ok($checkbox1.is(':disabled'), 'disables a checkbox');

    this.jobSelector.toggleFields(this.$fixture, '#bar', false);
    ok(!$checkbox2.is(':disabled'), 'enables a checkbox');
});

test('#isChecked', function () {
    ok(this.jobSelector.isChecked(this.$div1), 'returns true if the element has a checked checkbox');
    ok(!this.jobSelector.isChecked(this.$div2), 'returns false if the element does not have a checked checkbox');
});

test('#hasCheckedSelectField', function () {
    var div1 = this.$div1[0];
    var div2 = this.$div2[0];

    strictEqual(this.jobSelector.hasCheckedSelectField.call(div1), div1, 'returns the element if it has a checked checkbox');
    strictEqual(this.jobSelector.hasCheckedSelectField.call(div2), null, 'returns null if the element does not have a checked checkbox');
});

test('#hasNoCheckedSelectField', function () {
    var div1 = this.$div1[0];
    var div2 = this.$div2[0];

    strictEqual(this.jobSelector.hasNoCheckedSelectField.call(div2), div2, 'returns the element if it does not have a checked checkbox');
    strictEqual(this.jobSelector.hasNoCheckedSelectField.call(div1), null, 'returns null if the element has a checked checkbox');
});
