(function ($, BAMBOO) {

    BAMBOO.JENKINS_IMPORTER = {};

    BAMBOO.JENKINS_IMPORTER.Progressbar = function (options) {
        var that = this,
            finished = false;

        this.options = $.extend({
            classes: {
                progressbar: null
            },
            timeout: 1000
        }, options);

        this.updateTimeout = null;

        this.scheduleUpdate = function () {
            clearTimeout(that.updateTimeout);
            if (!finished) {
                that.updateTimeout = setTimeout(that.getProgress, that.options.timeout);
            }
        };
        this.getProgress = function () {
            return $.ajax({
                url: AJS.contextPath() + '/admin/jenkinsImport/getProgress.action',
                dataType: 'json',
                contentType: 'application/json',
                cache: false
            }).done(that.updateProgressbar).done(that.redirectOnCompletion).always(that.scheduleUpdate);
        };
        this.updateProgressbar = function (data) {
            $('.' + that.options.classes.progressbar).replaceWith(ji.page.importProgress.progressBar({
                current: data['chainsImported'] + 1,
                number: data['chainsToImport']
            }));
        };
        this.redirectOnCompletion = function (data) {
            if (data['importFinished']) {
                finished = true;
                window.location.replace(AJS.contextPath() + '/admin/jenkinsImport/importResultSummary.action');
            }
        };

        this.getProgress();
    };

    BAMBOO.JENKINS_IMPORTER.JobSelector = function (options) {
        var that = this;

        this.options = $.extend({
            classes: {
                table: null,
                jobRow: null,
                pipelineRow: null,
                enable: null,
                select: null
            }
        }, options);

        this.selectToggle = function (e) {
            var $this = $(this);
            var $row = $this.closest('.' + that.options.classes.jobRow + ', .' + that.options.classes.pipelineRow);
            var isChecked = $this.is(':checked');
            var fieldsToToggle = [
                'input[type="text"],',
                '.' + that.options.classes.enable
            ];
            var pipelineId, $pipelineJobRows;

            that.toggleFields($row, fieldsToToggle.join(','), !isChecked);
            if ($row.hasClass(that.options.classes.pipelineRow)) {
                pipelineId = $row.find('.' + that.options.classes.enable).val();
                $pipelineJobRows = $row.closest('.' + that.options.classes.table).find('.' + that.options.classes.jobRow).filter('[data-parent="' + pipelineId + '"]');
                that.toggleFields($pipelineJobRows.map(that.hasCheckedSelectField), fieldsToToggle.concat('.' + that.options.classes.select).join(','), !isChecked);
                that.toggleFields($pipelineJobRows.map(that.hasNoCheckedSelectField), '.' + that.options.classes.select, !isChecked);
            }
        };
        this.toggleFields = function ($rows, toggleSelectors, disabled) {
            $rows.find(toggleSelectors).prop('disabled', disabled);
        };
        this.hasCheckedSelectField = function () {
            return (that.isChecked(this) ? this : null);
        };
        this.hasNoCheckedSelectField = function () {
            return (!that.isChecked(this) ? this : null);
        };
        this.isChecked = function (el) {
            return !!$(el).has('.' + that.options.classes.select + ':checked').length;
        };
        this.init = function () {
            $(function () {
                $('.' + that.options.classes.table).on('change', '.' + that.options.classes.select, that.selectToggle);
            });
        };

        this.init();
    };

    BAMBOO.JENKINS_IMPORTER.IssueCollector = function (options) {
        var that = this;

        this.options = $.extend({
            trigger: null,
            description: null,
            containers: {
                feedbackSubmitConfirmation: null
            }
        }, options);

        this.submitFeedbackDone = function(data, textStatus, jqXHR) {

            AJS.messages.success(that.options.containers.feedbackSubmitConfirmation, {
                title: AJS.I18n.getText("jenkins.import.feedback.confirmation.success.title"),
                body: AJS.I18n.getText("jenkins.import.feedback.confirmation.success.body", data.key, data.url),
                closeable: false
            });
        };
        this.submitFeedbackFail = function(jqXHR, textStatus, errorThrown) {

            AJS.messages.error(that.options.containers.feedbackSubmitConfirmation, {
                title: AJS.I18n.getText("jenkins.import.feedback.confirmation.error.title"),
                body: textStatus,
                closeable: false
            });
        };
        this.setDialogToConfirmationPage = function() {
            that.dialog.gotoPage(2);
        };
        this.submitFeedback = function () {
            // show progress bar
            that.dialog.gotoPage(1);

            // hit REST endpoint
            $.ajax({
                type: 'POST',
                url: AJS.contextPath() + '/rest/ji/latest/submitFeedback/',
                data: {
                   description: $(that.options.description).val()
                },
                dataType: 'json'
            })
                .done(that.submitFeedbackDone)
                .fail(that.submitFeedbackFail)
                .always(that.setDialogToConfirmationPage);
        };

        this.init = function () {

            $(that.options.trigger).click(function(e) {
                e.preventDefault();

                that.dialog = new AJS.Dialog({ width: 600 })
                    // page 0 - form
                    .addHeader(AJS.I18n.getText("jenkins.import.feedback.title"))
                    .addPanel("feedbackForm", ji.page.importResult.feedbackSubmitForm({}))
                    .addButton(AJS.I18n.getText("global.buttons.submit"), function () { that.submitFeedback(); })
                    .addCancel(AJS.I18n.getText("global.buttons.close"), function () { that.dialog.remove(); })
                    // page 1 - progress
                    .addPage()
                    .addPanel("progress", ji.page.importResult.feedbackSubmitProgress({}))
                    // page 2 - confirmation
                    .addPage()
                    .addPanel("confirmation", ji.page.importResult.feedbackSubmitConfirmation({}))
                    .addCancel(AJS.I18n.getText("global.buttons.close"), function () { that.dialog.remove(); })
                    .gotoPage(0);

                that.dialog.show();
            });
        };

        this.init();
    };

}(jQuery, window.BAMBOO = (window.BAMBOO || {})));