AJS.$.namespace("AJS.gadget.bamboo");

/**
 * Returns a plan DOM object representing a plan with the status of
 *
 * @param plan - plan JSon object
 * @param builds - array of build result summary objects
 * @param baseUrl - baseUrl of Bamboo
 */
AJS.gadget.bamboo.generatePlanStatusDiv = function(plan, results, baseUrl) {
    var ICON_PLAN_BUILDING = '/images/jt/icn_building.gif',
        ICON_PLAN_PASSED = '/images/iconsv3/plan_successful_16.png',
        ICON_PLAN_FAILED = '/images/iconsv3/plan_failed_16.png',
        ICON_PLAN_CANCELLED = '/images/iconsv3/plan_canceled_16.png';

    var result,
        buildStatus;
    
    if (results.result) {
        result = results.result[0];
        buildStatus = result.state;
    }
    else {
        buildStatus = "Non executed";
    }
    // rotate the "allowed actions" array for easier access
    for (var action in plan.actions.action) {
        plan.actions.action[plan.actions.action[action].name] = true;
    }

    var divClass;
    if (buildStatus == "Successful") {
        divClass = 'build-result-successful';
    }
    else {
        if (buildStatus == "Failed") {
            divClass = 'build-result-error';
        }
        else {
            divClass = 'build-result-nonexecuted';
        }
    }

    var buildIconSrc = baseUrl;
    if (plan.isActive) {
        buildIconSrc += ICON_PLAN_BUILDING;
    } else if (buildStatus == "Successful") {
        buildIconSrc += ICON_PLAN_PASSED;
    }
    else {
        if (buildStatus == "Failed") {
            buildIconSrc += ICON_PLAN_FAILED;
        }
        else {
            buildIconSrc += ICON_PLAN_CANCELLED;
        }
    }

    var actions = {
        STOP: {
            href:  baseUrl + '/build/admin/stopPlan.action?planKey=' + plan.key,
            img:   baseUrl + '/images/iconsv3/stop.png',
            title: "Stop '" + plan.name + "'"
        },
        RUN: {
            href: baseUrl + '/build/admin/triggerManualBuild.action?buildKey=' + plan.key,
            img:  baseUrl + '/images/iconsv3/start.png',
            title: "Build '" + plan.name + "'"
        },
        EDIT: {
            href: baseUrl + '/build/admin/edit/editBuildConfiguration.action?buildKey=' + plan.key,
            img:  baseUrl + '/images/jt/icn_edit.gif',
            title: "Edit '" + plan.name + "'"
        }
    };

    var planStatusDiv;
    if (buildStatus != "Non executed") {
        planStatusDiv = AJS.$("<div/>").attr({id: plan.key}).addClass("build-result").addClass(divClass)
                .append(AJS.$("<div/>")
                .append(AJS.$("<div/>").addClass("build-info")
                .append(AJS.$("<img/>").attr({id: plan.key + "-buildIcon", src: buildIconSrc}))
            //                .append(AJS.$("<a/>").attr({id: plan.key + "-projectName", href: baseUrl + "/browse/" + plan.projectKey}).text(plan.projectName)).append("&nbsp;&ndash;&nbsp;")
                .append(AJS.$("<a/>").attr({id: plan.key + "-planName", href: baseUrl + "/browse/" + plan.key}).text(plan.name)).append("&nbsp;-&nbsp;")
                .append(AJS.$("<a/>").attr({id: plan.key + "-buildNumber", href: baseUrl + "/browse/" + result.key}).text(result.number))
                )
                .append(AJS.$("<div/>").attr({id: 'planOperations' + plan.key}).addClass("planOperations"))
                )
                .append(AJS.$("<div/>").addClass("build-details")
                .append(AJS.$("<span/>").attr({id: "lastBuilt" + plan.key, title: result.buildStartedTime})
                .append(AJS.$("<strong/>").text("Ran: "))
                .append(result.buildRelativeTime)
                )
                .append(AJS.$("<span/>").attr({id: "reason" + plan.key}).html(result.buildReason))
                .append(AJS.$("<span/>").attr({id: "duration" + plan.key}).addClass("last")
                .append(AJS.$("<strong/>").text("Duration: "))
                .append(result.buildDurationDescription)
                )
                )
                .append(AJS.$("<div/>").attr({id: 'recentBuilds' + plan.key}).addClass("build-history"));
    }
    else {
        planStatusDiv = AJS.$("<div/>").attr({id: plan.key}).addClass("build-result").addClass(divClass)
                .append(AJS.$("<div/>")
                .append(AJS.$("<div/>").addClass("build-info")
                .append(AJS.$("<img/>").attr({id: plan.key + "-buildIcon", src: buildIconSrc}))
                .append(AJS.$("<a/>").attr({id: plan.key + "-planName", href: baseUrl + "/browse/" + plan.key}).text(plan.name)).append("&nbsp;-&nbsp;")
                )
                .append(AJS.$("<div/>").attr({id: 'planOperations' + plan.key}).addClass("planOperations"))
                )
    }
    // add action icons depending on user's privileges
    if (plan.actions.action.BUILD) {
        if (plan.isActive) {
            AJS.$('#planOperations' + plan.key, planStatusDiv)
                    .append(AJS.$("<a/>").addClass("internalLink").attr({id: 'stopBuild' + plan.key, href: actions.STOP.href})
                    .append(AJS.$("<img/>").attr({title: actions.STOP.title, alt: actions.STOP.title, src: actions.STOP.img }))
                    );
        }
        else {
            AJS.$('#planOperations' + plan.key, planStatusDiv)
                    .append(AJS.$("<a/>").addClass("internalLink").attr({id: 'runBuild' + plan.key, href: actions.RUN.href})
                    .append(AJS.$("<img/>").attr({title: actions.RUN.title, alt: actions.RUN.title, src: actions.RUN.img }))
                    );
        }
    }
    if (plan.actions.action.ADMINISTRATION) {
        AJS.$('#planOperations' + plan.key, planStatusDiv)
                .append(AJS.$("<a/>").attr({id: 'editBuild' + plan.key, href: actions.EDIT.href})
                .append(AJS.$("<img/>").attr({title: actions.EDIT.title, alt: actions.EDIT.title, src: actions.EDIT.img }))
                );
    }

    if (result) {
        // add status of recent builds
        for (var i = results.result.length - 1; i >= 0; i--) {
            var recentBuild = results.result[i],
                buildResultHref = AJS.format('{0}/browse/{1}', baseUrl, recentBuild.key),
                buildResultClass = (recentBuild.state == "Successful") ? "success" : "error";

            AJS.$('#recentBuilds' + plan.key, planStatusDiv).append(AJS.$("<span/>").addClass(buildResultClass)
                    .append(AJS.$("<a/>").attr({href: buildResultHref, title: recentBuild.key})));
        }
    }

    return planStatusDiv;
};

AJS.gadget.bamboo.planStatusTemplate = [
    "<div class='build'>",
        "<div class='build-inside'>",
            "<div class='result_title'>",
                "<a class='build-name'></a>",
            "</div>",
            "<div class='result_plan'></div>",
            "<div class='result_vitals'></div>",
            "<div class='result_stats'></div>",
        "</div>",
    "</div>"
    ].join('');

AJS.gadget.bamboo.planStatusWallboardTemplate = [
    "<div class='builds'>",
        "<div class='indicator'>",
            "<h1 class='build-name'></h1>",
            "<p class='reason'></p>",
        "</div>",
        "<div class='commit'></div>",
     "</div>"
    ].join('');

AJS.gadget.bamboo.generateWallboardPlanStatusDiv = function (plan, results, baseUrl) {
    var key = plan.key,
        buildName = plan.name,
        result;
    if (results.result) {
        result = results.result[0];
    }

    var success = false,
        $box = AJS.$(AJS.gadget.bamboo.planStatusWallboardTemplate);

    $box.find('.build-name')
        .text(buildName);

    if (result.state == 'Successful') {
        $box.find('.indicator')
            .addClass('success')
            .removeClass('fail');
    } else {
        $box.find('.indicator')
            .addClass('fail')
            .removeClass('success');
    }

    if(key != "") {
        $box.append("<span>"+key+"</span>");
    }
    if(result.buildTestSummary != "") {
        $box.append("<span>"+result.buildTestSummary+"</span>");
    }
    if(result.buildRelativeTime != ""){
        $box.append("<span>"+result.buildRelativeTime+"</span>");
    }

    var reason = result.buildReason;
    if(reason === '') {
        $box.find('.build-name').css("border-bottom", "none").css("padding-bottom", "0");
        $box.find('.reason').text('').addClass("empty");
    } else {
        $box.find('.reason').html(reason);
    }

    return $box;
};

AJS.gadget.bamboo.planStatusDescriptor = function (args) {
    return  {
        action: "",
        theme : AJS.gadget.bamboo.getConfigTheme(gadgets),
        fields: AJS.gadget.bamboo.config.generatePlansConfigFields(
                baseUrl,
                gadget,
                args.plans,
        {})
                .concat([
            {
                userpref: AJS.gadget.bamboo.config.PREF_IS_CONFIGURED,
                type: "hidden",
                value: "true"
            }
        ])
    };
};

AJS.gadget.bamboo.planStatusAjaxOptions = function () {
    return {
        url: "/rest/api/latest/project.json",
        data:  {
            "expand": "projects.project.plans.plan",
            "max-results": "1000",
            "nocache": Math.floor(new Date().getTime())
        }
    };
};

AJS.gadget.bamboo.planStatusViewTemplateWallboard = function (baseUrl, gadget, args) {
    AJS.gadget.bamboo.planStatusGenericTemplate(baseUrl, gadget, args,
            (function() {gadget.getView().addClass("gadget-loaded");}),
            AJS.gadget.bamboo.generateWallboardPlanStatusDiv);

};

AJS.gadget.bamboo.planStatusViewTemplateNonWallboard = function (baseUrl, gadget, args) {
    AJS.gadget.bamboo.planStatusGenericTemplate(baseUrl, gadget, args,
            AJS.gadget.bamboo.generateFooter,
            AJS.gadget.bamboo.generatePlanStatusDiv);
};

AJS.gadget.bamboo.planStatusGenericTemplate = function (baseUrl, gadget, args, footerFunc, statusFunc ) {
    gadget.getView().empty();
    gadget.showLoading();

    AJS.$("a.internalLink").live("click", function() {
        var reqOptions = {
            url: this.href,
            async: false
        };

        AJS.$.ajax(reqOptions);

        window.setTimeout(function() {
            gadget.showView(true);
        }, 1000);
        return false;
    });

    AJS.gadget.bamboo.config.getSelectedPlans(gadget, {}, function(plans) {
        var plansProcessed = 0;

        if (plans.length == 0) {
            gadget.getView()
                    .append(AJS.gadget.bamboo.buildNoPlansSelectedDiv(gadget))
                    .append(AJS.gadget.bamboo.generateFooter(baseUrl));
            gadget.hideLoading();
            gadgets.window.adjustHeight();
        }
        else {
            var jsonResults = {};
            AJS.$(plans).each(function(i, planKey) {
                jsonResults[planKey] = {};
                var reqOptions = {
                    url: "/rest/api/latest/result/" + planKey + ".json",
                    data:  {
                        expand:        "results[0:9].result",
                        "max-results": 10,
                        nocache:       Math.floor(new Date().getTime())
                    },
                    success: function(response) {
                        jsonResults[planKey].results = response.results;

                        var reqOptions = {
                            url: "/rest/api/latest/plan/" + planKey + ".json",
                            data:  {
                                "expand": "actions",
                                "nocache": Math.floor(new Date().getTime())
                            },
                            success: function(response) {
                                jsonResults[planKey].plan = response;

                                if (++plansProcessed === plans.length) {
                                    AJS.$(plans).each(function(i, planKey) {
                                        // it is possible to get here object for oauth (success?), so additional check
                                        if (jsonResults[planKey].plan.key) {
                                            gadget.getView().append(statusFunc(jsonResults[planKey].plan, jsonResults[planKey].results, baseUrl));
                                        }
                                    });
                                    gadget.getView().append(footerFunc(baseUrl));
                                    gadget.hideLoading();
                                    gadgets.window.adjustHeight();
                                }
                            }
                        };
                        gadget.showLoading();

                        AJS.$.ajax(reqOptions);
                    }
                };
                gadget.showLoading();

                AJS.$.ajax(reqOptions);
            });
        }
    });
};