AJS.$.namespace("AJS.gadget.bamboo.config");

AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES = "useFavourites";
AJS.gadget.bamboo.config.PREF_QUICK_FIND = "_quickFind";
AJS.gadget.bamboo.config.PREF_PLANS = "planKeys";
AJS.gadget.bamboo.config.PREF_IS_CONFIGURED = "isConfigured";

var ICON_DELETE = '/images/delete.gif';

AJS.gadget.bamboo.config.groupBy = {
    MONTH:        "MONTH",
    WEEK:         "WEEK",
    DAY:          "DAY",
    AUTO:         "AUTO",
    BUILD_NUMBER: "BUILD_NUMBER"
};

AJS.gadget.bamboo.config.filter = {
    ALL:            "ALL",
    LAST_7_DAYS:    "LAST_7_DAYS",
    LAST_30_DAYS:   "LAST_30_DAYS",
    LAST_90_DAYS:   "LAST_90_DAYS",
    LAST_25_BUILDS: "LAST_25_BUILDS"
};


/**
 * Generate configuration field with hierarchical display of Bamboo plans
 *
 * @param gadget
 */
AJS.gadget.bamboo.config.generatePlansConfigFields = function(baseUrl, gadget, jsonPlans, filterOptions) {

    var showHidePlans = function(useFavourites, searchField, plans) {
            if (useFavourites) {
                searchField.parent().hide();
                plans.parent().hide();
            }
            else {
                searchField.parent().show();
                searchField.show();
                plans.parent().show();
                plans.show();
                searchField.focus();
            }
            gadgets.window.adjustHeight();
        },
        isNotProjectKey = function(token) { return token && token.charAt(0) != '_'; },
        isProjectKey = function(token) { return token && token.charAt(0) == '_'; };

    var options = [],
        useFavourites = {
            type: "checkbox",
            options: [{
                id: AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES,
                label: gadget.getMsg("gadget.config.plans.useMyFavourites.label"),
                selected: (gadget.getPref(AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES) == 'true')
            }],
            userpref: AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES,
            label: gadget.getMsg("gadget.config.plans.label"),
            value: gadget.getPref(AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES)
        },
        dummyPlan = {
            type: "hidden",
            userpref: AJS.gadget.bamboo.config.PREF_PLANS,
            value: ""
        };

    var planContainer = {
        id: "selected-plan-holder",
        type: "callbackBuilder", callback: function(parentDiv) {
            // Add the search field
            parentDiv.append(
                AJS.$("<div/>").attr("id", "quickfind-container").append(
                    AJS.$("<label/>").addClass("overlabel").attr({
                        "for": AJS.gadget.bamboo.config.PREF_QUICK_FIND,
                        id: "quickfind-label"
                    }).text(gadget.getMsg("gadget.config.plans.searchPlan.label"))
                ).append(
                    AJS.$("<input/>").attr("id", AJS.gadget.bamboo.config.PREF_QUICK_FIND)
                ).append(
                    AJS.$("<span/>").addClass("inline-error")
                ).append(
                    AJS.$("<span/>").addClass("description").text(gadget.getMsg("gadget.config.plans.searchPlan.description"))
                ).append(
                    AJS.$("<div/>").attr("id", "plan-list-container").append(
                            AJS.$("<span/>").attr("id", "noPlanSelectedText").text(gadget.getMsg("gadget.common.error.noPlansSelected"))
                         )
                )
            );

            AJS.gadget.fields.applyOverLabel("quickfind-label");

            var searchField = AJS.$("#" + AJS.gadget.bamboo.config.PREF_QUICK_FIND),
                plans = AJS.$("#selected-plan-holder");

            searchField.autocomplete("/rest/gadget/1.0/projectAndPlan.json", {
                dataType:"json",
                cacheLength: 1,
                formatItem: function(data, i, max, value, term) {
                    return i + " " + data.planName + " [" + data.planKey + "]";
                },
                formatResult: function(data) {
                    return data.planKey;
                },
                parse: function(data) {
                    var items = new Array();
                    for (var i = 0; i < data.length; i++) {
                        items[items.length] = { data:data[i], value:data[i].planKey, result:data[i].planName };
                    }
                    return items;
                },
                minChars: 0,
                width: 250,
                max: 25,
                extraParams: function () {
                    searchField.flushCache();
                    filterOptions.selectedPlans = function() {
                        var selectedPlansValue = AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val();
                        return AJS.$.grep(selectedPlansValue.split("|"), isNotProjectKey).join(",");
                    },
                    filterOptions.selectedProjects = function() {
                        var selectedPlansValue = AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val(),
                            projects = [];
                        AJS.$.each(AJS.$.grep(selectedPlansValue.split("|"), isProjectKey), function() {
                            projects.push(this.substr(1))
                        });
                        return projects.join(",");
                    };
                    return filterOptions;
                }(),
                autoFill: false,
                selectFirst: false,
                scroll: true,
                scrollHeight: 125
            }).result(function(event, data, formatted) {
                var d = {
                    // Projects are stored with a _ prefix e.g. "_BAM,CON-MAIN"
                    id: (data.type == 'project') ? '_' + data.planKey: data.planKey,
                    planKey: data.planKey,
                    planName: data.planName,
                    type: data.type
                };
                AJS.gadget.bamboo.config.addPlan(baseUrl, d);
                searchField.val("");
            });

            // handle useFavourite check and initial value
            AJS.$('#' + AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES).click(function() {
                showHidePlans(this.checked, searchField, plans);
            });

            showHidePlans(gadget.getPref(AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES) == 'true', searchField, plans);

            AJS.$('.remove-plan').live('click', function() {
                var planSpanHolder = AJS.$(this).closest('.plan-span-holder'),
                    planId = "";
                if (planSpanHolder.length === 1) {
                    planId = AJS.$(planSpanHolder).attr("id");
                    planSpanHolder.remove();
                }

                if (AJS.$("#plan-list-container .plan-span-holder").length === 0) {
                    AJS.$("#noPlanSelectedText").show();
                }

                var plansValue = AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val();
                plansValue = jQuery.grep(plansValue.split("|"), function(planToken) { return planToken != planId }).join("|");                
                AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val(plansValue);
            });

                        
            var prefs = gadget.getPrefArray(AJS.gadget.bamboo.config.PREF_PLANS);
            AJS.$(prefs).each(function() {
                // check if there are projects selected (projects are prefixed with _) 
                if (!(this.match("^_") == "_")) {
                    var planName = "",
                        planKey = this;
                    AJS.$(jsonPlans.projects.project).each(function() {
                        var project = this;
                        if (project.plans.plan) {
                            AJS.$(project.plans.plan).each(function() {
                                var localPlan = this;
                                if (localPlan.key == planKey) {
                                    planName = localPlan.name;
                                }
                            });
                        }
                    });

                    if (!planName) {
                        for (var i = 0, ii = jsonPlans.projects.project.length; i < ii; i++) {
                            var project = jsonPlans.projects.project[i];
                            if (project.plans.plan) {
                                for (var j = 0, jj = project.plans.plan.length; j < jj; j++) {
                                    var localPlan = project.plans.plan[j];
                                    if (localPlan.stages.stage) {
                                        for (var k = 0, kk = localPlan.stages.stage.length; k < kk; k++) {
                                            var stage = localPlan.stages.stage[k];
                                            if (stage.plans.plan) {
                                                for (var m = 0, mm = stage.plans.plan.length; m < mm; m++) {
                                                    var localPlan = stage.plans.plan[m];
                                                    if (localPlan.key == planKey) {
                                                        planName = localPlan.name;
                                                        break;
                                                    }
                                                }
                                                if (planName) {
                                                    break;
                                                }
                                            }
                                        }
                                        if (planName) {
                                            break;
                                        }
                                    }
                                }
                                if (planName) {
                                    break;
                                }
                            }
                        }
                    }

                    if (planKey) {
                        var optionPlan = {
                            id: planKey,
                            planKey: planKey,
                            planName: planName,
                            type: "plan"
                        };
                        AJS.gadget.bamboo.config.addPlan(baseUrl, optionPlan);
                    }
                }
                else {
                    var projectKey = this.substr(1, this.length - 1),
                        projectName = "";
                    AJS.$(jsonPlans.projects.project).each(function() {
                        var project = this;
                        if (project.key == projectKey)
                        {
                            projectName = project.name;
                        }
                    });

                    var optionPlan = {
                        id: this,
                        planKey: projectKey,
                        planName: projectName,
                        type: "project"
                    };
                    AJS.gadget.bamboo.config.addPlan(baseUrl, optionPlan);
                }
            });
        } // End callback
    };

    options.push(useFavourites);
    options.push(planContainer);
    options.push(dummyPlan);
    return options;
};

/**
 * Removes empty plan keys
 * @param plans
 */
function removeEmptyKeyFromTable(plans) {
    var tmp = [];
    for (var i = 0; i < plans.length; i++) {
        if (plans[i].length > 0) {
            tmp.length += 1;
            tmp[tmp.length - 1] = plans[i];
        }
    }
    return tmp;
}

/**
 * Removes duplicates in the array 'a'
 * @author Johan Känngård, http://dev.kanngard.net
 */
function unique(a) {
    var tmp = [];
    for (var i = 0; i < a.length; i++) {
        if (!contains(tmp, a[i])) {
            tmp.length += 1;
            tmp[tmp.length - 1] = a[i];
        }
    }
    return tmp;
}

/**
 * Returns true if 's' is contained in the array 'a'
 * @author Johan Känngård, http://dev.kanngard.net
 */
function contains(a, e) {
    for (var j = 0; j < a.length; j++)if (a[j] == e)return true;
    return false;
}

/**
 *
 * @param gadget
 * @param options  Options to extend
 * @param callback
 */
AJS.gadget.bamboo.config.getSelectedPlans = function(gadget, options, callback) {
    // get preferences for this specific key
    var prefs = gadget.getPrefArray(AJS.gadget.bamboo.config.PREF_PLANS);

    if (gadget.getPref(AJS.gadget.bamboo.config.PREF_PLANS_USE_FAVOURITES) == 'true') {
        var ajaxOptions = {
            url: "/rest/api/latest/project.json",
            data:  {
                "expand": "projects.project.plans.plan.stages.stage.plans.plan",
                "favourite": "true",
                "max-results": "1000",
                "nocache": Math.floor(new Date().getTime())
            },
            success: function(response) {
                var plans = [];
                AJS.$(response.projects.project).each(function() {
                    var project = this;
                    if (project.plans.plan) {
                        AJS.$(project.plans.plan).each(function() {
                            var plan = this;
                            plans.push(plan.key);
                        });
                    }
                });
                if (AJS.$.isFunction(callback)) {
                    callback.call(gadget, plans);
                }
            }
        };

        AJS.$.ajax(AJS.$.extend(true, ajaxOptions, options));
    }
    else {
        var projects = AJS.$.grep(prefs, function(planKey){
                return (planKey.match(/^_/));
            }),
            plansOnly = AJS.$.grep(prefs, function(planKey){
                return !(planKey.match(/^#/) || (planKey.match(/^_/)));
            });

        if (projects.length > 0) {
            var projectNames = [ ];
            projects = AJS.$(projects).each(function() {
                projectNames.push(this.substring(1));
            });
                        
            var ajaxOptions = {
                url: "/rest/gadget/latest/projectAndPlan/plansForProjects.json",
                data:  {
                    "projects": projectNames,
                    "max-results": "1000",
                    "nocache": Math.floor(new Date().getTime())
                },
                success: function(response) {
                    var plans = plansOnly;

                    AJS.$(response).each(function() {
                        var plan = this;
                        // if oauth response is returned
                        if (typeof plan.planKey != "undefined") {
                            plans.push(plan.planKey);
                        }
                    });     
                    if (AJS.$.isFunction(callback)) {
                        plans = unique(plans);
                        plans = removeEmptyKeyFromTable(plans);
                        callback.call(gadget, plans);
                    }
                }
            };

            AJS.$.ajax(AJS.$.extend(true, ajaxOptions, options));
        }
        else {
            if (AJS.$.isFunction(callback)) {
                plansOnly = unique(plansOnly);
                plansOnly = removeEmptyKeyFromTable(plansOnly);
                callback.call(gadget, plansOnly);
            }
        }
    }
};

AJS.gadget.bamboo.config.buildPlanSpan = function(baseUrl, id, planKey, planName, type) {
    var newPlan = "<ul class='plan-span-holder' id='" + id + "'>";
    newPlan += "<li class='plan'>";
    newPlan += "<a class='remove-plan'><img title='Remove' alt='Remove' src='" + baseUrl + ICON_DELETE + "'/></a>";
    newPlan += "<a title='Open in Bamboo' href='" + baseUrl + "/browse/"+ planKey + "'>" + planName  + "</a>";
    newPlan += "</li></ul>";
    return newPlan;
};

AJS.gadget.bamboo.config.addPlan = function(baseUrl, data) {
    var plansValue = AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val(),
        plans = plansValue.split("|");
    if (contains(plans, data.id) == false) {
        var newPlan = AJS.gadget.bamboo.config.buildPlanSpan(baseUrl, data.id, data.planKey, data.planName, data.type);
        if (plansValue != "") {
            plansValue += "|";
        }
        plansValue += data.id;
        AJS.$("input[name='" + AJS.gadget.bamboo.config.PREF_PLANS + "']").val(function() {
            return plansValue;
        });

        AJS.$("#noPlanSelectedText").hide();
        AJS.$("#plan-list-container").append(newPlan);
    }
    gadgets.window.adjustHeight();
};

AJS.gadget.bamboo.config.generateSelectPlan = function(gadget, prefsKey, jsonPlans) {
    var field = {
        type: "select",
        userpref: prefsKey,
        label: gadget.getMsg("gadget.config.plan.label"),
        selected: gadget.getPref(prefsKey),
        options : function() {
            var options = [ ];

            AJS.$(jsonPlans.plans.plan).each(function() {
                var plan = this;
                options.push({value: plan.key, label: plan.name});
            });

            return options;
        }()
    };

    return field;
};


AJS.gadget.bamboo.config.generateSelectGroupByPeriod = function(gadget, prefsKey) {
    var field = {
        type: "select",
        userpref: prefsKey,
        label: gadget.getMsg("gadget.config.groupBy.label"),
        selected: gadget.getPref(prefsKey),
        options: [
            {
                value: AJS.gadget.bamboo.config.groupBy.MONTH,
                label: gadget.getMsg("gadget.config.groupBy.month")
            },
            {
                value: AJS.gadget.bamboo.config.groupBy.WEEK,
                label: gadget.getMsg("gadget.config.groupBy.week")
            },
            {
                value: AJS.gadget.bamboo.config.groupBy.DAY,
                label: gadget.getMsg("gadget.config.groupBy.day")
            },
            {
                value: AJS.gadget.bamboo.config.groupBy.AUTO,
                label: gadget.getMsg("gadget.config.groupBy.auto")
            }
        ]
    };

    return field;
};

AJS.gadget.bamboo.config.generateSelectGroupByPeriodAndBuildNumber = function(gadget, prefsKey) {
    var field = AJS.gadget.bamboo.config.generateSelectGroupByPeriod(gadget, prefsKey);
    field.options.push({
        value: AJS.gadget.bamboo.config.groupBy.BUILD_NUMBER,
        label: gadget.getMsg("gadget.config.groupBy.buildNumber")
    });

    return field;
};


AJS.gadget.bamboo.config.generateSelectDateFilter = function(gadget, prefsKey) {
    var field = {
        type: "select",
        userpref: prefsKey,
        label: gadget.getMsg("gadget.config.range.label"),
        selected: gadget.getPref(prefsKey),
        options: [
            {
                value: AJS.gadget.bamboo.config.filter.ALL,
                label: gadget.getMsg("gadget.config.range.allBuilds")
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_7_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 7)
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_30_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 30)
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_90_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 90)
            }
        ]
    };

    return field;
};

AJS.gadget.bamboo.config.generateSelectDateAndBuildNumberFilter = function(gadget, prefsKey) {
    var field = {
        type: "select",
        userpref: prefsKey,
        label: gadget.getMsg("gadget.config.range.label"),
        selected: gadget.getPref(prefsKey),
        options: [
            {
                value: AJS.gadget.bamboo.config.filter.ALL,
                label: gadget.getMsg("gadget.config.range.allBuilds")
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_7_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 7)
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_30_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 30)
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_90_DAYS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNDays"), 90)
            },
            {
                value: AJS.gadget.bamboo.config.filter.LAST_25_BUILDS,
                label: AJS.format(gadget.getMsg("gadget.config.range.lastNBuilds"), 25)
            }
        ]
    };

    return field;
};

// From JIRA used to overlay fields, pretty cool...
AJS.$.namespace("AJS.gadget.fields");
AJS.gadget.fields.applyOverLabel = function(overLabelId){
    AJS.$("#" + overLabelId).each(function (){
        var label = AJS.$(this).removeClass("overlabel").addClass("overlabel-apply").click(function(){
            AJS.$("#" + AJS.$(this).attr("for")).focus();
        }),
        field = AJS.$("#" + label.attr("for")).focus(function(){
            label.hide();
        }).blur(function(){
            if (AJS.$(this).val() === ""){
                label.show();
            }
        });
        if (field.val() !== ""){
            label.hide();
        }
    });
};

